﻿using System;
using System.Text;

namespace ACLAS
{
    public static class CovTool
    {
        public static String HexString(Byte[] buf, int index, int len)
        {
            StringBuilder sb = new StringBuilder();
            for (int i = 0; i < len; i++)
                sb.Append(buf[index + i].ToString("X2"));
            return sb.ToString();
        }

        public static String RespString(String title , Byte[] buf, int len)
        {
            StringBuilder sb = new StringBuilder(title);
            for (int i = 0; i < len; i++)
                sb.Append(buf[i].ToString("X2")+" ");
            return sb.ToString()+"\r\n";
        }

        public static int toInt(Byte[] buf, int index)
        {
            int val = 0;
            for (int i = 0; i < 4; i++)
            {
                val <<= 8;
                val |= buf[index + 3 - i];
            }
            return val;
        }

        public static uint toUInt(Byte[] buf, int index)
        {
            uint val = 0;
            for (int i = 0; i < 4; i++)
            {
                val <<= 8;
                val |= buf[index + 3 - i];
            }
            return val;
        }
        
        public static void IntToByte(int val , Byte[] buf, int idx)
        {
            for (int i = 0; i < 4; i++)
            {
                buf[idx + i] = (Byte) val;
                val >>= 8;
            }
        }

        public static void UIntToByte(UInt32 val, Byte[] buf, int idx)
        {
            for (int i = 0; i < 4; i++)
            {
                buf[idx + i] = (Byte)val;
                val >>= 8;
            }
        }

        public static void UInt16ToByte(UInt16 val, Byte[] buf, int idx)
        {
                buf[idx]   = (Byte) val;
                buf[idx+1] = (Byte) (val >> 8);
        }

        public static String toString(Byte[] buf, int index, int len)
        {
            StringBuilder s = new StringBuilder();

            for (int i = 0; i < len; i++)
            {
                Byte b = buf[index + i];
                if (b >= 0x20)
                    s.Append((Char)b);
            }
            return s.ToString();
        }

        public static String toString(String title, Byte[] buf, int index, int len)
        {
            StringBuilder s = new StringBuilder(title);

            for (int i = 0; i < len; i++)
            {
                Byte b = buf[index + i];
                if (b >= 0x20)
                    s.Append((Char)b);
            }
            return s.ToString();
        }

        private static Byte Hex(char c)
        {
            if (c >= '0' && c <= '9')
                return (Byte) (c - '0');

            if (c >= 'A' && c <= 'F')
                return (Byte)(c - ('A' - 10));

            throw new Exception();
        }

        private static Byte Hex2Byte(char h, char l)
        {
            return  (Byte) ((Hex(h) << 4) | Hex(l));
        }

        public static Byte[] HexStringToByte(String s)
        {
            s = s.Trim().ToUpper(); 
            int len = s.Length;
            if (len == 0 || ((len & 1) != 0))
            {
                return null;
            }
            Byte[] buf = new Byte[len / 2];

            for (int i=0,j=0 ;i<len;i+=2,j++)
            {
                try
                {
                    buf[j] = Hex2Byte(s[i], s[i + 1]);
                }
                catch
                {
                    return null;
                }
            }
            return buf;
        }

        public static Byte[] Str2Byte(String s)
        {
            Byte [] buf = null;
            if (s.Length > 0)
            {
                buf = new Byte[s.Length];
                for (int i = 0; i < s.Length; i++)
                    buf[i] = (byte)s[i];
            }
            return buf;
        }

        public static Byte[] MakeValue(Byte Sector , Byte Block , int Value)
        {
            Byte Addr = BlockAddr(Sector, Block); 
            Byte[] buf = new Byte[16]; 
            IntToByte(Value , buf, 0);
            IntToByte(~Value, buf, 4);
            IntToByte(Value, buf, 8);
            buf[12] = buf[14] = Addr;
            buf[13] = buf[15] = (Byte)~Addr;
            return buf; 
        }

        public static Byte BlockAddr(Byte Sector, Byte Block)
        {
            return (Byte) ((Sector << 2) + Block);
        }

        public static bool GetValue(Byte Sector, Byte Block, ref int Value , Byte[] buf)
        {
            Byte Addr = BlockAddr(Sector, Block);

            if (Addr != buf[12] || Addr != buf[14])
                return false;
            Addr = (Byte) ~Addr;
            if (Addr != buf[13] || Addr != buf[15])
                return false;

            int Value1 = toInt(buf,0);
            int Value2 = ~toInt(buf, 4);
            int Value3 = toInt(buf, 8);

            if ((Value1 == Value2) && (Value1 == Value3))
            {
                Value = Value1;
                return true;
            }

            return false;
        }

        public static Boolean StringCompare(String s, String d)
        {
            int len = d.Length;
            if (s.Length < len) return false;
            return (s.Substring(0, len).CompareTo(d) == 0);
        }

    }
}
