﻿using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace ACLAS
{
    public static class DebugClass
    {
        static System.Windows.Forms.Timer tmr;
        static Label lbMem;
        static uint lastmem;
        static bool m_Debug = false;

        public static void ShowLog(string msg)
        {
            if (m_Debug)
            {
                WINAPI.DebugMsg(msg); 
            }
        }

        public static bool Debug
        {
            set
            {
                m_Debug = value;
            }
            get
            {
                return m_Debug;
            }
        }
        
        public static void Init(Label lb)
        {
            lbMem = lb;

            tmr = new System.Windows.Forms.Timer();
            tmr.Tick += new EventHandler(tmr_Tick);
            tmr.Interval = 200;
            lastmem = 0;
            tmr.Enabled = true;
        }

        public static void Close()
        {
            tmr.Enabled = false;
        }

        static void tmr_Tick(object sender, EventArgs e)
        {
            AvailPhys();
        }

        public struct MEMORYSTATUS
        {
            public UInt32 dwLength;
            public UInt32 dwMemoryLoad;
            public UInt32 dwTotalPhys;
            public UInt32 dwAvailPhys;
            public UInt32 dwTotalPageFile;
            public UInt32 dwAvailPageFile;
            public UInt32 dwTotalVirtual;
            public UInt32 dwAvailVirtual;
        }

        [DllImport("CoreDll.dll")]
        public static extern void GlobalMemoryStatus
        (
            ref MEMORYSTATUS lpBuffer
        );

        [DllImport("CoreDll.dll")]
        public static extern int GetSystemMemoryDivision
        (
            ref UInt32 lpdwStorePages,
            ref UInt32 lpdwRamPages,
            ref UInt32 lpdwPageSize
        );

        public static void AvailPhys()
        {
            UInt32 storePages = 0;
            UInt32 ramPages = 0;
            UInt32 pageSize = 0;
            uint mem;

            // 強制立即執行層代零至指定層代的記憶體回收。
            GC.Collect();
            // 暫止目前的執行緒，直到處理完成項佇列的執行緒已經清空該佇列為止。
            GC.WaitForPendingFinalizers();

            int res = GetSystemMemoryDivision(ref storePages,
                ref ramPages, ref pageSize);

            // Call the native GlobalMemoryStatus method
            // with the defined structure.
            MEMORYSTATUS memStatus = new MEMORYSTATUS();
            GlobalMemoryStatus(ref memStatus);

            mem = memStatus.dwAvailPhys;

            if (lastmem != mem)
            {
                lastmem = mem;
                lbMem.Text = mem.ToString();
            }
        }
    }
}
