﻿using System;
using System.Runtime.InteropServices;

namespace ACLAS
{
    public enum FILE_ACCESS : byte
    {
        ANY = 0,
        READ = 1,
        WRITE = 2
    }

    public enum FILE_METHOD : byte
    {
        BUFFERED   =  0,
        IN_DIRECT  =  1,
        OUT_DIRECT =  2,
        NEITHER    =  3
    }
    
    public enum FILE_DEVICE : ushort 
    {
        FILE_STREAM        = 0x1e,
        //
        // Windows CE Specific Defines
        //
        HAL                = 0x101,
        CONSOLE            = 0x102,
        PSL                = 0x103,
        SERVICE            = 0x104,
        NLED               = 0x105,
        NOTIFY             = 0x106,
        GPS                = 0x107,
        SQM                = 0x108,
        USERDRIVER         = 0x109,
        REFLECTOR          = 0x10a,
        CE_DMA             = 0x110,
        FLASH              = 0x111,
        CONNMGR_SERVICE    = 0x112,
        BACKLIGHT          = 0x113,
        SENSOR_COMMON_MDD  = 0x114,
        PARTITION          = 0x115,
        STORE              = 0x116,
        ULDR               = 0x117,
        TOUCH              = 0x118,
        LIGHT              = 0x119
    }

    // 裝置類別 by Jacky
    public class Device
    {
        [DllImport("coredll.dll")]
        public static extern bool WriteFile(
        System.IntPtr h_comm,
        Byte[] text,
        int length,
        ref   int length2,
        int overlap
        );

        [DllImport("coredll.dll")]
        public static extern bool ReadFile(
        System.IntPtr h_comm,
        Byte[] text,
        int length,
        ref int length2,
        int overlap
        );

        [DllImport("coredll.dll")]
        public static extern bool DeviceIoControl(
        System.IntPtr h_comm,
        int code,
        Byte[] inBuf,
        int InBufSize,
        Byte[] outBuf,
        int outBufSize,
        ref int BytesReturned,
        int overlap
        );

        [DllImport("coredll.dll")]
        public static extern System.IntPtr CreateFile(
        string FileName,                                         //   file   name 
        uint DesiredAccess,                                   //   access   mode 
        uint ShareMode,                                           //   share   mode 
        uint SecurityAttributes,                         //   Security   Attributes 
        uint CreationDisposition,                       //   how   to   create 
        uint FlagsAndAttributes,                         //   file   attributes 
        int hTemplateFile                                       //   handle   to   template   file 
        );

        [DllImport("coredll.dll")]
        public static extern bool CloseHandle(System.IntPtr h_comm);

        [DllImport("coredll.dll")]
        private static extern uint SetDevicePower(
        String Device,
        uint flag,
        uint mode
        );

        private String DeviceName;
        private IntPtr m_Handle;
        private uint DeviceMode;
        private bool isOpen = false;

        public IntPtr Handle
        {
            get
            {
                return m_Handle; 
            }
        }

        public Device(String name, uint mode)
        {
            DeviceName = name;
            DeviceMode = (uint)mode;
        }

        public void ShutdownPower()
        {
            SetDevicePower(DeviceName, WINAPI.POWER_NAME, (uint)POWER_STATE.D4);
        }

        public Boolean IOControl(int code, Byte[] inbuf, int insize, Byte[] obuf, int osize, ref int rsize)
        {
            rsize = 0;
            return DeviceIoControl(m_Handle, code, inbuf, insize, obuf, osize, ref rsize, 0);
        }

        public Boolean Open()
        {
            if (isOpen)
            {
                Close();
            }
            m_Handle = CreateFile(DeviceName, DeviceMode, 0, 0, WINAPI.OPEN_EXISTING, 0, 0);
            isOpen = ((int)m_Handle != WINAPI.INVALID_HANDLE_VALUE);
            return isOpen;
        }

        public Boolean Close()
        {
            isOpen = false;
            return CloseHandle(m_Handle);
        }

        public bool Write(Byte[] buf, int size)
        {
            int rsize = 0;
            return WriteFile(m_Handle, buf, size, ref rsize, 0);
        }

        public int Read(Byte[] buf, int size)
        {
            int rsize = 0;
            ReadFile(m_Handle, buf, size, ref rsize, 0);
            return rsize;
        }

        public static uint CTL_CODE(FILE_DEVICE DeviceType, ushort Function, FILE_METHOD Method, FILE_ACCESS Access) 
        {
            return (uint) (((ushort)DeviceType << 16) | ((byte)Access << 14) | (Function << 2) | (byte)Method);
        }
    }
}
