﻿using System;
using System.Threading;

namespace ACLAS
{
    public class MagClass
    {
        private Device MagCard = null;
        private Thread Magthd = null;
        private Byte[] MagData = new Byte[1024];
        private int len1;
        private int len2;
        private int len3;
        private String m_Mag1,m_Mag2,m_Mag3;

        public event EventHandler OnMagCardRead = null;

        public MagClass()
        {
            // 打開驅動裝置類別
            MagCard = new Device("MAG1:", WINAPI.GENERIC_READ);
        }

        public void MakeMagHeader(Byte[] buf)
        {
            len1 = CovTool.toInt(buf, 0);
            len2 = CovTool.toInt(buf, 4);
            len3 = CovTool.toInt(buf, 8);

            if (len1 > 0)
                m_Mag1 = CovTool.toString(buf, 12, len2);
            else
                m_Mag1 = null;

            if (len2 > 0)
                m_Mag2 = CovTool.toString(buf, 140, len2);
            else
                m_Mag2 = null;

            if (len3 > 0)
                m_Mag3 = CovTool.toString(buf, 240, len2);
            else
                m_Mag3 = null;
        }

        public Boolean Open()
        {
            Boolean rc;

            rc = MagCard.Open();
            if (rc)
            {
                Magthd = new Thread(MagProc);
                Magthd.Start();
            }
            return rc;
        }

        public void Close()
        {
            if (Magthd != null)
            {
                Magthd.Abort();
                Thread.Sleep(200);
            }
            MagCard.Close();
        }

        private void MagProc()
        {
            Boolean go = true;
            int len;

            while (go)
            {
                len = MagCard.Read(MagData, MagData.Length);
                if (len != 0)
                {
                    MakeMagHeader(MagData);

                    if (OnMagCardRead != null)
                        OnMagCardRead.Invoke(this, null);
                }
                Thread.Sleep(100);
            }
        }

        public String Mag1Data
        {
            get
            {
                return m_Mag1;
            }
        }

        public String Mag2Data
        {
            get
            {
                return m_Mag2;
            }
        }

        public String Mag3Data
        {
            get
            {
                return m_Mag3;
            }
        }
    }
}
