﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using System.Diagnostics;

namespace ACLAS
{
    public static class MediaPlay
    {
        [DllImport("coredll")]
        private static extern IntPtr SendMessage(IntPtr hWnd, uint Msg, int wParam, uint lParam);

        private static Thread thdPlay = null;

        const int WM_COMMAND = 0x111;

        const int WA_INIT = 40000;
        const int WA_PLAY = 40001;
        const int WA_PAUSE = 40002;
        const int WA_STOP = 40003;
        const int WA_START = 40004;
        const int WA_BIG = 40005;
        const int WA_SMALL = 40006;

        private static IntPtr hWnd = IntPtr.Zero;

        private static ManualResetEvent PlayEvent = new ManualResetEvent(false);

        private static MediaList m_list = new MediaList();
        
        private static int PlayIndex = 0;

        public static string MediaPath = "";

        delegate void TextBoxCallBack(string msg);
        delegate void ClearTextBoxCallBack();

        private static bool m_BigSize = true;

        private static Rectangle BigWindow = new Rectangle(0, 60, 1024, 668);
        private static Rectangle SmallWindow = new Rectangle(0, 60, 512, 334);

        private static IntPtr SendMsg(uint Msg, int wParam, uint lParam)
        {
            IntPtr rc = IntPtr.Zero;
            try
            {
                if (hWnd != IntPtr.Zero)
                {
                    rc = SendMessage(hWnd, Msg, wParam, lParam);
                }
            }
            catch
            {
            }
            return rc;
        }

        public static void Open()
        {
            thdPlay = new Thread(PlayerProc);
            thdPlay.Start();

           // Thread.Sleep(100); // Wait Thread Run...
        }

        public static void Close()
        {
            if (thdPlay != null)
            {
                thdPlay.Abort();
                Thread.Sleep(100);
            }

            StopMedia();

                        
        }

        public static void SetBigWindow(int x, int y, int w, int h)
        {
            BigWindow = new Rectangle(x, y, w, h);
        }

        public static void SetSmallWindow(int x, int y, int w, int h)
        {
            SmallWindow = new Rectangle(x, y, w, h);
        }

        // 增加項目到撥放清單中
        public static void AddMedia(string fn)
        {
            m_list.Add(fn);
            PlayEvent.Set();
        }

        // 移除播放清單中的項目
        public static void RemoveMedia(string fn)
        {
                m_list.Remove(fn);
                if (m_list.Count == 0)
                    PlayEvent.Reset();
        }

        // 清除所有播放清單
        public static void ClearAllMedia()
        {
            m_list.Clear();
            PlayEvent.Reset();
        }

        public static void PauseMedia()
        {
            SendMsg(WM_COMMAND, WA_PAUSE, 0);
        }

        public static void PlayMedia()
        {
            EnableVideoArea();
            SendMsg(WM_COMMAND, WA_PLAY, 0);
        }

        public static void StopMedia()
        {
            SendMsg(WM_COMMAND, WA_STOP, 0);
        }

        public static bool BigSize
        {
            get
            {
                return m_BigSize;
            }
        }

        public static void SetMediaSize(bool big)
        {
            m_BigSize = big;

            EnableVideoArea();

            SendMsg(WM_COMMAND, big ? WA_BIG : WA_SMALL, 0);
        }

        public static void BigMedia()
        {
            SetMediaSize(true);
        }

        public static void SmallMedia()
        {
            SetMediaSize(false);
        }

        public static void StartMedia()
        {
            EnableVideoArea();

            SendMsg(WM_COMMAND, WA_START, (uint)(m_BigSize ? 1 : 0));
        }

        public static void VideoArea(int x, int y, int width, int height, Color col)
        {
            using (Bitmap bmp = new Bitmap(width, height, System.Drawing.Imaging.PixelFormat.Format16bppRgb565))
            {
                using (Graphics gr = Graphics.FromImage(bmp))
                {
                    gr.Clear(col);
                }
                TVout.TvDrawImage(bmp, x, y, width, height);
            }
        }

        public static void EnableVideoArea()
        {
            int x, y, w, h;

            if (m_BigSize)
            {
                x = BigWindow.X;
                y = BigWindow.Y;
                w = BigWindow.Width;
                h = BigWindow.Height;
            }
            else
            {
                x = SmallWindow.X;
                y = SmallWindow.Y;
                w = SmallWindow.Width;
                h = SmallWindow.Height;
            }

            VideoArea(x, y, w, h, Color.FromArgb(0, 0, 1));
        }

        private static void PlayerProc()
        {
            string msg;

            WINAPI.SetThreadCPU(2);

            hWnd = IntPtr.Zero;

            while (true)
            {
                PlayEvent.WaitOne();

                int cnt = m_list.Count;  

                if (cnt == 0)
                {
                    PlayEvent.Reset();
                    continue;
                }

                if (PlayIndex >= cnt)
                {
                   PlayIndex = 0;
                }

                // 播放

                hWnd = TVout.PlayVideo(MediaPath + "\\" + m_list.GetList(PlayIndex));

                TVout.SetVideoWindow(hWnd, true, BigWindow.X, BigWindow.Y, BigWindow.Width, BigWindow.Height);
                TVout.SetVideoWindow(hWnd, false, SmallWindow.X, SmallWindow.Y, SmallWindow.Width, SmallWindow.Height);

                StartMedia();

                TVout.WaitForPlay();  

                WINAPI.DebugMsg("Play Done!!!!"); 
                
                PlayIndex++;

            }
        }

    }


    public class MediaList
    {
        // 撥放清單
        private List<string> m_list = new List<string>();
        private object m_Lock = new object();

        public string GetList(int idx)
        {
            lock (m_Lock)
            {
                return m_list[idx];  
            }
        }

        public void Add(string s)
        {
            lock (m_Lock)
            {
                m_list.Add(s); 
            }
        }

        public void Remove(string item)
        {
            lock (m_Lock)
            {
                m_list.Remove(item);  
            }
        }

        public int Count
        {
            get
            { 
                int c;
                lock (m_Lock)
                {
                    c = m_list.Count;  
                }
                return c;
            }
        }

        public void Clear()
        {
            lock (m_Lock)
            {
                m_list.Clear();  
            }
        }
    }


}
