﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;
using System.Threading; 
using System.Windows.Forms;
 
namespace ACLAS
{
    public class SmartCard
    {
        private int Slot;
        private Device SCard;
        private int ATRlen;
        private Byte[] ATRbuf = new Byte[33];
      
        private SC_PARA para = new SC_PARA();
        private const int MAX_CMD_LEN = 512;

        private Byte[] Resp = new Byte[MAX_CMD_LEN];

        private TextBox tbDebug = null;

        private Thread  thrd = null;

        public  Boolean bInitSuccess;
        public int conv;

        private Boolean CardChange = false;
        private Boolean CardExist = false;

        SC_ATR [,] ibyte = new SC_ATR[4,4];
        /* 7816-3 1997 Table 7, 8 */
        private int []  Ftab = new int[]
        { 
           372, 372, 558,  744, 1116, 1488, 1860, -1,
		    -1, 512, 768, 1024, 1536, 2048,   -1, -1
        };

        private int [] Dtab = new int[]
        {
          -1, 1, 2, 4, 8, 16, 32, -1, 12, 20, -1, -1, -1, -1, -1, -1
        };

        private UInt16[] SoundOK = new UInt16[] {
            1 , 3400, 100 , 40 
        };

        private UInt16[] SoundBad = new UInt16[] {
            3 , 3000, 80 , 40 
        };

        private UInt16[] SoundReady = new UInt16[] {
            2 , 1500, 60 , 40 ,
            1 , 2500, 200 , 40 
        };

        delegate void TextCallBack(String msg);

        private Boolean sc_GetStatus()
        {
            UInt32 code=0;
            Boolean rc = Cmd(Command.GETSTATUS, ref code);
            if (rc)
            {
                if ((code & 1) != 0)
                    CardChange = true;
                CardExist = ((code & 0x80000000) != 0);
            }
            return rc;
        }

        private void MainLoop()
        {
            Boolean found = false;

            while (true)
            {
                sc_GetStatus(); 
                if (CardChange)
                {
                    if (!found)
                    {
                        if (CardExist)
                        {
                            found = true;
                            PDEBUG("Smart Card Insert!\r\n");
                            if (ReadSmartCard()== RetCode.OK)
                            {
                                Buzzer.SoundOut(SoundOK);   
                            }
                            else
                            {
                                PDEBUG("Smart Card Read Error!\r\n"); 
                                Buzzer.SoundOut(SoundBad);   
                            }
                        }
                    }
                    else
                    {
                        if (!CardExist)
                        {
                            PDEBUG("Smart Card Remove!\r\n"); 
                            found = false;
                            Buzzer.SoundOut(SoundReady);   
                        }
                    }
                    CardChange = false;
                }
                else
                    Thread.Sleep(100);  
            }
        }

        private void ClearATR()
        {
            for (int i = 0; i < 4; i++)
                for (int j = 0; j < 4; j++)
                    ibyte[i, j] = new SC_ATR();
        }

        public void SetDebugTextBox(TextBox tb)
        {
            tbDebug = tb;
        }

        public SmartCard(int no)
        {
            Slot = no;
            SCard = new Device("SAM" + Slot + ":", WINAPI.GENERIC_WRITE | WINAPI.GENERIC_READ);   // 打開驅動裝置類別
        }

        public Boolean Open()
        {
            Boolean rc = SCard.Open();
            if (rc)
            {
               thrd = new Thread(MainLoop);
               thrd.Start(); 
            }
            return rc;
        }

        public void Close()
        {
            if (thrd != null)
                thrd.Abort();  
            SCard.Close();
        }

        public Boolean Cmd(Command cmd)
        {
            int rsize = 0;
            return (SCard.IOControl((int)cmd, null, 0, null, 0, ref rsize));
        }

        public Boolean Cmd(Command cmd, ref UInt32 RetCode)
        {
            int rsize = 0;
            Boolean rc;
            Byte[] buf = new Byte[4];

            rc = SCard.IOControl((int)cmd, null, 0, buf, sizeof(UInt32), ref rsize);
            RetCode = (UInt32)CovTool.toInt(buf, 0);
            return rc;
        }

        public Boolean Cmd(Command cmd,  UInt32 val)
        {
            int rsize = 0;
            Byte[] buf = new Byte[4];
            CovTool.UIntToByte(val, buf, 0);  
            return  SCard.IOControl((int)cmd, buf , 4, null, 0 , ref rsize);
        }

        public Boolean Cmd(Command cmd, ref int size, Byte[] buf)
        {
            int rsize = 0;
            Boolean rc;
            rc = SCard.IOControl((int)cmd, null, 0, buf, size, ref rsize);
            size = rsize;
            return rc;
        }

        public Boolean Cmd(Command cmd, int size, Byte[] buf)
        {
            int rsize = 0;
            return SCard.IOControl((int)cmd, buf, size, null, 0, ref rsize);
        }


        private void ShowLog(String msg)
        {
            if (tbDebug.InvokeRequired)
            {
                TextCallBack cb = new TextCallBack(ShowLog);
                tbDebug.Invoke(cb, new Object[] { msg });
            }
            else
            {
                tbDebug.Text += msg;
                tbDebug.Select(tbDebug.TextLength, 0);
                tbDebug.ScrollToCaret();
            }
        }

        private  void PDEBUG(String msg)
        {
            if (tbDebug != null)
                ShowLog(msg); 
        }

        private RetCode sc_T0_SendCommand(Byte[] buf, int len, Byte[] recv, ref int count)
        {
            Byte INS = buf[1];
            int lc, le, local_count;
            RetCode rc = RetCode.CMD;
            Byte[] pbytebuf = new Byte[1];
            Byte pbyte;

            /* check for incorrect length */

            if (len < 5 || len > MAX_CMD_LEN)
                return rc;

            if (len != 5 && len != (buf[4] + 5) && len != (buf[4] + 6))
                return rc;

            lc = 0; le = 0;			/* pick lc, le from command string */

            if (len == 5)
            {
                if (buf[4] != 0)
                    le = buf[4];
            }
            else
            {
                if (len == (buf[4] + 5))
                    lc = buf[4];
                else
                {
                    lc = buf[4] + 1;
                    le = buf[len - 1];
                    if (le == 0)			/* if le appears and it is equal to 0, it means
									max length(256) of data is needed */
                        le = 256;
                }
            }

            local_count = 5;

            count = 0;

            sc_ClearFIFO();

            /* begin writting 5 bytes command header */
            if ((rc = sc_RawWrite(buf, local_count)) != RetCode.OK)
            {
                PDEBUG("raw write failed " + rc.ToString() + "\r\n");
                return RetCode.WRITE;
            }

            do
            {
                /* get procedure byte */
                do
                {
                    if ((rc = sc_RawRead(pbytebuf, 1)) != RetCode.OK)
                    {
                        PDEBUG("raw read failed " + rc.ToString() + "\r\n");
                        return RetCode.READ;
                    }
                    pbyte = pbytebuf[0];
                } while (pbyte == 0x60);

                if (pbyte == (INS ^ 0x01) || pbyte == INS)
                {
                    if (lc != 0)
                        local_count = lc;
                    else
                        local_count = le;
                }
                else if ((INS ^ 0xfe) == pbyte || (INS ^ 0xff) == pbyte)
                {

                    local_count = 1;

                }
                else if ((pbyte & 0xf0) == 0x60 || (pbyte & 0xf0) == 0x90)
                {
                    recv[count++] = pbyte;		/* save SW1*/

                    if ((rc = sc_RawRead(pbytebuf, 1)) != RetCode.OK)	/* read SW2 */
                        return rc;
                    pbyte = pbytebuf[0];
                    recv[count++] = pbyte;	/* save SW2 */

                    /* wait for card turning into receiving mode */
                    Thread.Sleep(1);

                    return RetCode.OK;
                }
                else
                {
                    PDEBUG("Unknown procedure byte : " + pbyte.ToString("X2") + "\r\n");
                    return RetCode.READ;
                }

                /* read/write card according lc, le and local_count */
                if (lc != 0)
                {

                    /* if write, a delay is a must before write so that card can have time to
                        enter receiving mode. At least 100us */
                    Thread.Sleep(1);

                    if ((rc = sc_RawWrite(buf, len - lc, local_count)) != 0)
                    {
                        PDEBUG("Write Error : " + rc.ToString() + "\r\n");
                        return RetCode.WRITE;
                    }
                    lc -= local_count;
                }
                else
                {
                    if ((rc = sc_RawRead(recv, count, local_count)) != RetCode.OK)
                    {
                        PDEBUG("Read Error : " + rc.ToString() + "\r\n");
                        return RetCode.READ;
                    }
                    le -= local_count;
                    count += local_count;
                }

                if (lc < 0 || le < 0)
                    return RetCode.READ;

            } while (true);
        }

        private RetCode sc_SendCommand(Byte[] send, int len, Byte[] recv, ref int count)
        {
            RetCode rc;

            rc = sc_TestCardReady();

            if (rc != RetCode.OK)
            {
                PDEBUG("zzn card is no ready \r\n");
                return rc;
            }

            rc = sc_T0_SendCommand(send, len, recv, ref count);

            return rc;
        }


        public int sc_ApduCommand(Byte cla, Byte ins, int p1, int p2, int lc, Byte[] data, int le, Byte[] resp)
        {
            Byte[] buf = new Byte[MAX_CMD_LEN];
            int i = 0;
            int rc = 0;

            buf[i++] = cla;
            buf[i++] = ins;
            if (p1 >= 0 && p1 <= 255)
                buf[i++] = (Byte)p1;
            if (p2 >= 0 && p2 <= 255)
                buf[i++] = (Byte)p2;
            if (lc >= 0 && lc <= 255)
            {
                buf[i++] = (Byte)lc;
                for (int j = 0; j < lc; j++)
                    buf[i++] = data[j];
            }
            if (le >= 0 && le <= 255)
                buf[i++] = (Byte)le;

            if (sc_SendCommand(buf, i, resp, ref rc) != RetCode.OK)
                rc = 0;

            return rc;
        }

        public RetCode ReadSmartCard()
        {
            int rc;
            Byte[] file_id;

            if (sc_TestCardReady() != RetCode.OK)
                return RetCode.BAD;

            if (sc_ColdReset() != RetCode.OK)
                return RetCode.BAD;

            rc = sc_GetChallenge8();

            if (rc == 0)
                return RetCode.BAD;
            
            PDEBUG(CovTool.RespString("Challenge : ",Resp ,rc));

            file_id = new Byte[] { 0x3f, 0x00 };

            rc = sc_SelectFile(file_id, Resp);

            if (rc == 0)
                return RetCode.BAD; 

            PDEBUG(CovTool.RespString("Select File (3F00) : ", Resp, rc));

// 尚待完成
#if false
            file_id = new Byte[] { 0x00, 0x05 };

            rc = sc_SelectFile(file_id, Resp);

            if (rc == 0)
                return RetCode.BAD;

            PDEBUG(CovTool.RespString("Select File (0003) : ", Resp, rc));

            rc = sc_ReadBinary_SFI( 5, 1, 5, Resp);

            if (rc == 0)
                return RetCode.BAD;

            PDEBUG(CovTool.RespString("Read ID File : ", Resp, rc));
#endif

            return RetCode.OK;  
        }

        /* Standard Command */
        public int sc_SelectFile(Byte [] file_id, Byte []  resp)
        {
            return sc_ApduCommand( 0, 0xa4, 0, 0, 2, file_id, -1, resp);
        }

        public int sc_ReadBinary_SFI(Byte SFI, Byte pos, Byte len, Byte [] resp)
        {
            return sc_ApduCommand(0, 0xb0, (SFI & 0x1f) | 0x80, pos, -1, null , len, resp);
        }

        public int sc_GetChallenge8() // 盤問卡片
        {
            return sc_ApduCommand(0, 0x84, 0, 0, -1, null , 8, Resp);
        }

        private void sc_DeactivateCard()
        {
            Cmd(Command.DEACTIVATE);
        }

        private void sc_PowerOff()
        {
            Cmd(Command.POWEROFF);
        }

        private Boolean sc_PowerOnMode(PowerClass pl)
        {
            UInt32 code;
            Byte[] buf = new Byte[4];
            switch (pl)
            {
                case PowerClass.OPERATIONCLASS_A_5V:
                    code = 1;
                    break;
                case PowerClass.OPERATIONCLASS_B_3V3:
                    code = 0;
                    break;
                default:
                    return false;
            }
            CovTool.UIntToByte(code, buf, 0);
            return Cmd(Command.POWERON, 4, buf);
        }

        private void sc_Delay(int dly)
        {
            Thread.Sleep(dly);
        }

        private void sc_ActivateCard()
        {
            int len = 0;
            Cmd(Command.ACTIVATE);
            len = ATRbuf.Length;
            Cmd(Command.GETATR, ref len, ATRbuf);
            ATRlen = len;
        }

        private Boolean sc_MatchReader(UInt32 fi, UInt32 di)
        {
            UInt32  tmp;

            tmp = (((fi << 16) & 0xffff0000) | (di & 0x0000ffff));

            return Cmd(Command.MATCHREADER , tmp);
        }

        private Boolean sc_SetReaderPara()
        {
           return   Cmd(Command.SETPARAMETER ,para.Length , para.toByte() );
        }

        private RetCode sc_RawWrite(Byte[] buf, int count)
        {
            //	if (conv )	/* if need, convert byte */
            //		sc_ConvertString(p, *count);

            if (!SCard.Write(buf, count))
                return RetCode.WRITE;
            return RetCode.OK;
        }

        private RetCode sc_RawWrite(Byte[] buf, int idx , int count)
        {
            Byte [] nbuf = new Byte[buf.Length - idx];
            Array.Copy(buf, idx, nbuf, 0, buf.Length - idx);
            return sc_RawWrite( nbuf, count);
        }

        private RetCode sc_RawRead(Byte[] buf, int count)
        {
            int rc;

            rc = SCard.Read(buf, count);

            //	if(dev->conv)
            //		sc_ConvertString(buf, *count);

            if (rc != count)
                return RetCode.READ;

            return RetCode.OK;
        }

        private RetCode sc_RawRead(Byte[] buf, int idx, int count)
        {
            Byte [] rbuf = new Byte[count]; 

            RetCode rc;

            rc = sc_RawRead(rbuf, count);

            if (rc !=  RetCode.OK)
                return RetCode.READ;

            Array.Copy(rbuf, 0, buf, idx, count);    

            return RetCode.OK;
        }



        /* clear FIFO buffer of reception */
        private void sc_ClearFIFO()
        {
            Cmd(Command.CLEARFIFO);
        }

        // Protocol type selection
        private RetCode sc_DoPTS(int f, int d, int t)
        {
            Byte[] buf = new Byte[4];
            Byte[] rbuf = new Byte[4];
            int len = 4;
            RetCode rc;

            /* default envirment , pts not need */
            if ((t == 0 ||  f == 1) && d == 1)
                return RetCode.OK; 

            buf[0] = 0xff;  // PTSS
            buf[1] = (Byte)(0x10 | (t & 0x0f));  // PTS0
            buf[2] = (Byte)(((f << 4) & 0xf0) | (d & 0x0f));  // PTS1
            buf[3] = (Byte)(buf[0] ^ buf[1] ^ buf[2]);  // PCK

            sc_ClearFIFO();
/*
            PDEBUG("Sending PTS : " + buf[0].ToString("X2") + " "
                                           + buf[1].ToString("X2") + " "
                                           + buf[2].ToString("X2")+"\r\n");
*/
            if ((rc = sc_RawWrite(buf, len)) != RetCode.OK)
            {
                PDEBUG("Write error :" + rc.ToString() + "\r\n");
                return rc;
            }

            if ((rc = sc_RawRead(rbuf, len)) != RetCode.OK)
            {
                PDEBUG("Read error : " + rc.ToString() + "\r\n");
                return rc;
            }

            rc = RetCode.UNSUPPORTEDCARD;
/*
            PDEBUG("Received PTS : " + rbuf[0].ToString("X2") + " "
                                            + rbuf[1].ToString("X2") + " "
                                            + rbuf[2].ToString("X2")+"\r\n");
*/

            if (rbuf[0] != buf[0])
                return rc;

            if ((rbuf[1] & 0x0f) == (buf[1] & 0x0f) &&		/* success */
                ((rbuf[1] & 0xf0) == 0x10 || (rbuf[1] & 0xf0) == 0x00))
            {
                PDEBUG("PTS Request Success, now baudrate is " + (9600 * Dtab[d]) + "\r\n");
                return RetCode.OK;
            }

            return rc;
        }

        private RetCode sc_WarmReset()
        {
            RetCode rc;

            sc_DeactivateCard();

            Thread.Sleep(10);	/*delay 10ms */

            sc_ActivateCard();

            rc = sc_ATRCheck();

            return rc;
        }

        private RetCode sc_ATRCheck()
        {
            int i, h;
            Byte td, ts;
            int idx = 0;
            Boolean tck;

            /* invalid length of atr */
            if (ATRlen < 2)
                return RetCode.UNSUPPORTEDCARD;

            ts = ATRbuf[idx++];

            conv = 0;

            if (ts == 0x03)
            {
                conv = 1;
                // sc_ConvertString(dev->atr, dev->atr_len);
            }

            if (ts != 0x3b && ts != 0x3f)
                return RetCode.UNSUPPORTEDCARD;		/* unknow convention */

            td = ATRbuf[idx++];
            tck = false;
            h = td & 0x0f;

            while (true)
            {
                if ((td & 0x10) != 0)
                    idx++;
                if ((td & 0x20) != 0)
                    idx++;
                if ((td & 0x40) != 0)
                    idx++;
                if ((td & 0x80) != 0)
                {
                    td = ATRbuf[idx++];
                    tck = ((td & 0x0f) != 0);
                }
                else
                    break;

                if ((idx + h) > ATRlen)
                {
                    PDEBUG("ATR length check failed\r\n");
                    return RetCode.UNSUPPORTEDCARD;
                }
            }

            if (tck)
            {
                td = 0;
                for (i = 1; i < ATRlen; i++)
                    td ^= ATRbuf[i];
                if (td != 0)
                    return RetCode.UNSUPPORTEDCARD;
            }
            String s = "ATR CODE : ";
            for (i = 0; i < ATRlen; i++)
                s += ATRbuf[i].ToString("X2") + " ";
            PDEBUG(s + "\r\n");
            return RetCode.OK;
        }

        // i-Cash Card ATR    : 3B F7 18 00 00 81 71 FE 42 00 00 63 95 31 05 90 00 F1
        // 1st Bank visa Card : 3b e8 00 00 81 31 fe 45 00 73 c8 40 00 00 90 00 88

        private RetCode sc_ParseATR()  // 分析 ATR 表格
        {
            RetCode rc = RetCode.OK;
            int i;
            Byte td;
            Byte n, t, f, d;
            int idx = 1;
            Byte hb_len, hbyte;

            ClearATR();

            td = ATRbuf[idx++];
            hb_len = (Byte)(td & 0x0f);

            i = 0;

            while (i < 8)
            {
                /* TAi present */
                if ((td & 0x10) != 0)
                {
                    ibyte[i, (int)ATRIB.TAI].data = ATRbuf[idx++];
                    ibyte[i, (int)ATRIB.TAI].present = true;
                }

                /* TBi present */
                if ((td & 0x20) != 0)
                {
                    ibyte[i, (int)ATRIB.TBI].data = ATRbuf[idx++];
                    ibyte[i, (int)ATRIB.TBI].present = true;
                }

                /* TCi present */
                if ((td & 0x40) != 0)
                {
                    ibyte[i, (int)ATRIB.TCI].data = ATRbuf[idx++];
                    ibyte[i, (int)ATRIB.TCI].present = true;
                }

                /* TDi present */
                if ((td & 0x80) != 0)
                {
                    // PDEBUG("Protocol : "+ (ATRbuf[idx] & 0x0f) + "\r\n");
                    ibyte[i, (int)ATRIB.TDI].data = ATRbuf[idx++];
                    ibyte[i, (int)ATRIB.TDI].present = true;
                    td = ibyte[i, (int)ATRIB.TDI].data;
                    i++;
                }
                else
                    break;

            }

            hbyte = ATRbuf[idx];

            f = d = 1;  // Default value

            /* set f and d */
            if (ibyte[0, (int)ATRIB.TAI].present)
            {
                f = (Byte)((ibyte[0, (int)ATRIB.TAI].data >> 4) & 0x0f);
                d = (Byte)(ibyte[0, (int)ATRIB.TAI].data & 0x0f);
            }

            n = 0;

            if (ibyte[0, (int)ATRIB.TCI].present)
                n = (Byte)ibyte[0, (int)ATRIB.TCI].data;

            t = 0;

            if (ibyte[1 , (int)ATRIB.TDI].present)
            {	/* in specific mode */
                t = (Byte)(ibyte[1, (int)ATRIB.TDI].data & 0x0f);  // Protocol

             //   if ((ibyte[0 , (int)ATRIB.TAI].data & 0x10) != 0)
             //   {
             //       f = 0; d = 1;		/* default value */
             //   }

                if (!sc_MatchReader((UInt32)Ftab[f], (UInt32)Dtab[d]))
                {
                    if ((ibyte[1, (int)ATRIB.TDI].data & 0x80) == 0)
                    {	/* capable to change mode */
                        rc = sc_WarmReset();	/* change to negotiable mode */
                        if (rc != RetCode.OK)
                            return rc;

                        f = 0; d = 1;t = 0;

                        rc = sc_DoPTS(f, d, t);	/* parameter and protocol select */
                        if (rc != RetCode.OK)
                            return rc;
                    }
                    else
                        return RetCode.UNSUPPORTEDCARD;
                }
            }
            else
            {
                /* whatever reader support or not , keep in default mode */
                f = 0; d = 1; // t = 0;
            }

            para.fi = (UInt32) Ftab[f];
            para.di = (UInt32) Dtab[d];
            para.t = t;
            para.n = n;

            /* calculate cwt, bwt, etu, etc... */
            if (t != 0)
            {
                /* t = 1, not implemented yet */
                PDEBUG("Error! Only supported T = 0 protocol\r\n");
                return RetCode.UNSUPPORTEDCARD;
            }
            else
            {	/* t = 0 , or others */
                int wi = 12;

                if (ibyte[1,(int)ATRIB.TCI].present)
                    wi += ibyte[1,(int)ATRIB.TCI].data;

                para.bwt = para.cwt =(UInt32) (960 * wi * para.fi / 372);
            }

            if (sc_DoPTS( f, d, t) != 0)
                return RetCode.READ;

            if (!sc_SetReaderPara())
                return RetCode.UNSUPPORTEDCARD; 

            bInitSuccess = true;

            return  RetCode.OK;
        }

        private RetCode sc_ColdReset()
        {
            RetCode rc = RetCode.OK;

            sc_DeactivateCard();
            sc_PowerOff();

            /* delay 10 ms */
            sc_Delay(10);

            sc_PowerOnMode(PowerClass.OPERATIONCLASS_B_3V3);

            sc_ActivateCard();

            rc = sc_ATRCheck();

            if (rc != RetCode.OK)
                return rc;

            rc = sc_ParseATR();

            return rc;
        }

        public RetCode sc_TestCardReady()
        {
            sc_GetStatus();

            if (!CardExist)
                return RetCode.CARDREMOVED;
            else
                return RetCode.OK;
        }

        public enum RetCode
        {
            /* only used by library */
            OK = 0,
            CARDREMOVED = (0x01 | 0x80),
            CMD = (0x02 | 0x80),
            LIB_KEY = (0x03 | 0x80),
            UNSUPPORTEDCARD = (0x04 | 0x80),
            DEVICEBUSY = (0x05 | 0x80),
            READ = (0x06 | 0x80),
            WRITE = (0x07 | 0x80),
            CARDCHANGED = (0x08 | 0x80),
            BAD = (0xff)
        }

        public enum PowerClass
        {
            /* iso7816 operation class */
            OPERATIONCLASS_A_5V = 1,
            OPERATIONCLASS_B_3V3 = 2
        }

        public enum Command
        {
            POWERON = 0,	   	    /* card power on */
            POWEROFF = 1,			/* card power off */

            /* this two command is for power saving */
            POWERUP = 2,			/* card power up */
            POWERDOWN = 3,			/* card power down */

            ACTIVATE = 4,			/* activate card and save ATR to buffer,you can use GETATR to retrieve */

            DEACTIVATE = 5,			/* deactivate */
            C4C8READ = 6,	    	/* for general use */
            C4C8WRITE = 7,
            SETPARAMETER = 8,    	/* set reader parameter to adjust card */

            CLEARFIFO = 9,			/* clear reception buffer */
            GETSTATUS = 10,		    /* check card present */
            SELECTCARD = 11,		/* for multi-card  use */
            GETATR = 12,
            GETERRORNO = 13,		/* get error number */
            MATCHREADER = 14,		/* check whether the reader support the baudrate or not */

            SETPARITY = 15,
            SETCLOCK = 16
        }

        /* ATRIB = A(nswer) T(o) R(eset) I(nterface) B(yte) */
        private enum ATRIB
        {
            TAI = 0,
            TBI = 1,
            TCI = 2,
            TDI = 3,
        }
    }

    public class  SC_ATR
    {
        public Boolean present=false;
        public Byte data=0;
    }

    public class SC_PARA
    {
        public UInt32 n, t, cwt, bwt, fi, di;
        private Byte[] buf = new Byte[24];
        public Byte [] toByte()
        {
            CovTool.UIntToByte(n, buf, 0);
            CovTool.UIntToByte(t, buf, 4);
            CovTool.UIntToByte(cwt, buf, 8);
            CovTool.UIntToByte(bwt, buf, 12);
            CovTool.UIntToByte(fi, buf, 16);
            CovTool.UIntToByte(di, buf, 20);   
            return buf;
        }

        public int Length
        {
            get
            {
                return 24;
            }
        }
    }
}
