﻿using System;
using System.Runtime.InteropServices;
using System.Drawing;
using System.Drawing.Imaging;
using System.Threading;
using System.IO;
using Microsoft.Win32;

namespace ACLAS
{
    public static class TVout
    {
        public enum DisplayType
        {
            LCD1280x800 = 1,
            LCD1024x768 = 2,
            LCD800x600  = 3,
            LCD1366x768 = 4
        }

        private const String DllName = "TvOut.dll";

        private static Size m_size = new Size(1024, 768);

        private static int m_mqx=0, m_mqy=300, m_mqw=480, m_mqh=32 , m_mqfsz = 20 ;

        private static int m_adx = 0, m_ady = 0, m_adw = 480, m_adh = 260 , m_add =3;
        private static Color m_adc = Color.YellowGreen; 

        [DllImport(DllName)]
        private static extern uint InitTvOut();

        [DllImport(DllName)]
        private static extern Boolean DrawImage(IntPtr hbm, int x, int y, int Width, int Height, int sx, int sy, TvOutDrawMode mode);

        // 2013-7-3 設置 video 播放窗體位置及大小
        [DllImport(DllName)]
        public static extern void SetVideoWindow(IntPtr hWnd , bool big , int x, int y, int Width, int Height);

        [DllImport(DllName)]
        private static extern void DeInit();

        [DllImport(DllName)]
        public static extern IntPtr PlayVideo(string fn);

        [DllImport(DllName)]
        public static extern void WaitForPlay();
                
        private static Bitmap[] Msgbmp = null;
        private static Graphics marqeegr = null;
        private static Bitmap marqueebmp = null;
        private static int MsgCount;
        private static Thread marqueethd = null;
        private static Thread ADthd = null;
        private static String ADPath;

        private static String[] m_TestMsg;
        private static string m_FontName;
        private static Color m_BackColor, m_ForeColor;
        
        private static ManualResetEvent ADPlayEvent = new ManualResetEvent(false);

        public static Size GetCusDisplaySize()
        {
            Size sz = new Size(1280,800);

            try
            {
                RegistryKey di1 = Registry.LocalMachine.OpenSubKey(@"Drivers\Display\DDIPU\DI1", false);
                DisplayType dt = (DisplayType)Convert.ToUInt32(di1.GetValue("PanelType", 0));

                switch (dt)
                {
                    case DisplayType.LCD800x600:
                        sz = new Size(800, 600); 
                        break;
                    case DisplayType.LCD1024x768:
                        sz = new Size(1024, 768); 
                        break;
                    case DisplayType.LCD1280x800:
                        sz = new Size(1280, 800); 
                        break;
                    case DisplayType.LCD1366x768:
                        sz = new Size(1366, 768);
                        break;
                }
            }
            catch
            {
            }
            return sz;
        }

        public static void TvDrawImage(Bitmap bmp, int x, int y, int width, int height)
        {
            DrawImage(bmp.GetHbitmap(),x, y, width, height, 0, 0, TvOutDrawMode.SRCCOPY);
        }

        public static void TvDrawImage(Bitmap bmp, int x, int y, int width, int height ,TvOutDrawMode mode)
        {
            DrawImage(bmp.GetHbitmap(), x, y, width, height, 0, 0, mode);
        }

        public static void SetSize(int width,int height)
        {
            m_size = new Size(width, height); 
        }

        public static Boolean Open()
        {
            return (InitTvOut()==0);
        }

        public static void Close()  
        {
            if (marqueethd != null)
                marqueethd.Abort();
            if (ADthd != null)
                ADthd.Abort();

            MediaPlay.VideoArea(0, 0, m_size.Width, m_size.Height ,Color.Black);    

            DeInit();
        }

        public static void InitAD(String path)
        {
            ADPath = path;
            ADthd = new Thread(ADPlay);
            ADthd.Start(); 
        }

        public static void RunADPlay()
        {
            ADPlayEvent.Set();  
        }

        public static void StopADPlay()
        {
            ADPlayEvent.Reset();  
        }

        public static void SetupADPlay(int x, int y, int w, int h, Color col , int sec)
        {
            m_adx = x;
            m_ady = y;
            m_adw = w;
            m_adh = h;
            m_adc = col;
            m_add = (sec==0) ? 1 : sec;
        }

        // 廣告 AD
        private static void ADPlay()
        {
            WINAPI.SetThreadCPU(3);

            Bitmap sbmp = new Bitmap(480, 260, PixelFormat.Format16bppRgb565);
            Graphics gr = Graphics.FromImage((Image)sbmp);
            Brush bb = new SolidBrush(m_adc);

            DirectoryInfo di = new DirectoryInfo(ADPath);
            FileInfo[] fi = di.GetFiles();

            Bitmap[] ADbmp = new Bitmap[fi.Length];

            while (true)
            {
                for (int idx = 0; idx < fi.Length; idx++)
                {
                    ADPlayEvent.WaitOne();  

                    if (ADbmp[idx] == null)
                    {
                        ADbmp[idx] = new Bitmap(fi[idx].FullName);
                    }

                    gr.FillRectangle(bb, 0, 0, 480, 260);
                    gr.DrawImage((Image)ADbmp[idx], 5, 5);
                    TvDrawImage(sbmp, m_adx , m_ady , m_adw, m_adh);
                    Thread.Sleep(m_add * 1000);
                }
            }
        }

        public static void InitMarquee(String[] TestMsg , String FontName  , Color BackColor , Color ForeColor) 
        {
            int MsgWidth, MsgHeight;

            MsgCount = TestMsg.Length;
 
            Msgbmp = new Bitmap[MsgCount];

            Font fn = new Font( FontName , m_mqfsz , FontStyle.Regular);

            Brush fb = new SolidBrush(BackColor);
            Brush ff = new SolidBrush(ForeColor);

            marqueebmp = new Bitmap(m_mqw , m_mqh , PixelFormat.Format16bppRgb565);

            marqeegr = Graphics.FromImage((Image)marqueebmp);

            marqeegr.Clear(BackColor);
            TvDrawImage(marqueebmp, m_mqx, m_mqy, m_mqw, m_mqh);
            
            for (int i = 0; i < TestMsg.Length; i++)
            {
                SizeF sz = marqeegr.MeasureString(TestMsg[i], fn);

                MsgWidth = (int)sz.Width + m_mqw * 2;
                MsgHeight = m_mqh;

                Msgbmp[i] = new Bitmap(MsgWidth, MsgHeight, PixelFormat.Format16bppRgb565);

                using (Graphics gr = Graphics.FromImage((Image)Msgbmp[i]))
                {
                    gr.FillRectangle(fb, 0, 0, MsgWidth, MsgHeight);
                    gr.DrawString(TestMsg[i], fn, ff, m_mqw , 2);
                }
            }

            fn.Dispose();
            fb.Dispose();
            ff.Dispose();
        }

        public static void InitMarquee(int x, int y, int width, int height,int fontsize ,String[] TestMsg, String FontName, Color BackColor, Color ForeColor)
        {
            m_mqx = x;
            m_mqy = y;
            m_mqw = width;
            m_mqh = height;
            m_mqfsz = fontsize;

            m_TestMsg = TestMsg;
            m_FontName = FontName;
            m_ForeColor = ForeColor;
            m_BackColor = BackColor;

            marqueethd = new Thread(marquee);
            marqueethd.Start(); 
        }

        private static void marquee()
        {
            int MsgIdx = 0;
            int MsgRow = 0;

            WINAPI.SetThreadCPU(3);

            InitMarquee(m_TestMsg, m_FontName, m_BackColor, m_ForeColor);
            
            while (true)
            {
                MsgIdx = 0;
                while (true)
                {
                    marqeegr.DrawImage(Msgbmp[MsgRow], new Rectangle(0 , 0, m_mqw, Msgbmp[MsgRow].Height), new Rectangle(MsgIdx, 0, m_mqw, Msgbmp[MsgRow].Height), GraphicsUnit.Pixel);

                    TvDrawImage(marqueebmp, m_mqx , m_mqy , m_mqw , m_mqh);

                    MsgIdx++;
                    if ((MsgIdx + m_mqw) > Msgbmp[MsgRow].Width)
                    {
                        if (++MsgRow >= MsgCount) MsgRow = 0;
                        break;
                    }
                    Thread.Sleep(20);
                }
            }
        }
    }

    public enum TvOutDrawMode : uint
    {
        SRCCOPY = 0x00CC0020, /* dest = source                   */
        SRCPAINT = 0x00EE0086, /* dest = source OR dest           */
        SRCAND = 0x008800C6, /* dest = source AND dest          */
        SRCINVERT = 0x00660046, /* dest = source XOR dest          */
        SRCERASE = 0x00440328, /* dest = source AND (NOT dest )   */
        NOTSRCCOPY = 0x00330008, /* dest = (NOT source)             */
        NOTSRCERASE = 0x001100A6, /* dest = (NOT src) AND (NOT dest) */
        MERGECOPY = 0x00C000CA, /* dest = (source AND pattern)     */
        MERGEPAINT = 0x00BB0226, /* dest = (NOT source) OR dest     */
        PATCOPY = 0x00F00021, /* dest = pattern                  */
        PATPAINT = 0x00FB0A09, /* dest = DPSnoo                   */
        PATINVERT = 0x005A0049, /* dest = pattern XOR dest         */
        DSTINVERT = 0x00550009, /* dest = (NOT dest)               */
        BLACKNESS = 0x00000042, /* dest = BLACK                    */
        WHITENESS = 0x00FF0062  /* dest = WHITE                    */
    }

    //  TV out 客顯 交易窗口

    public class PayWindow
    {
        private Bitmap bmp;
        private Graphics gr;
        private Brush bb = new SolidBrush(Color.DarkSlateBlue);
        private Pen pen = new Pen(Color.Blue);
        private const String UseFontName = "Arail";

        public String[] Text = new String[2];
        private Font payfn3 = new Font(UseFontName, 26, FontStyle.Bold);
        private Font payfn1 = new Font(UseFontName, 20, FontStyle.Bold);
        private Font payfn2 = new Font(UseFontName, 16, FontStyle.Bold);
        Brush paybr = new SolidBrush(Color.White);
        private int Minute;
        public int WelcomeWaitCnt = 0;
        private int m_width = 480;

        private System.Windows.Forms.Timer tmr = new System.Windows.Forms.Timer();

        private void Init()
        {
            bmp = new Bitmap(m_width , 60 , PixelFormat.Format16bppRgb565);
            gr = Graphics.FromImage((Image)bmp);
            gr.FillRectangle(bb, new Rectangle(0, 0, m_width , 60));
            gr.DrawRectangle(pen, 5, 3, m_width - 7 , 54);
            tmr.Tick += new EventHandler(tmr_Tick);
        }

        public PayWindow()
        {
            Init();
        }

        public PayWindow(int width)
        {
            m_width = width;
            Init();
        }
        
        public void Start()
        {
            tmr.Interval = 1000;
            Welcome();
            ShowWelcome();
        }

        public void Clear()
        {
            gr.FillRectangle(bb, new Rectangle(6, 4, m_width - 9 , 52));
        }

        void tmr_Tick(object sender, EventArgs e)
        {
            DateTime now = DateTime.Now;

            if (WelcomeWaitCnt > 0)
            {
                WelcomeWaitCnt--;

                if (WelcomeWaitCnt == 0)
                {
                    Minute = now.Minute;
                    ShowWelcome();
                }
            }
            else
            {
                if (now.Minute != Minute)
                {
                    Minute = now.Minute;
                    ShowWelcome();
                }
            }
        }

        public void Welcome()
        {
            tmr.Enabled = true;
            Minute = 255;
        }

        public void ShowWelcome()
        {
            String s;

            Clear();

            s = "Now" + DateTime.Now.ToString(" yyyy-MM-dd HH:mm");

            gr.DrawString("Welcome" , payfn1, paybr, 10, 6);

            gr.DrawString(s, payfn2, paybr, 10, 34);

            TVout.TvDrawImage(bmp, 0, 0, m_width , 60);
        }

        public void ShowPay()
        {
            tmr.Enabled = false;

            Clear();

            if (Text[0].Length > 0)
            {
                if (Text[1].Length > 0)
                {
                    gr.DrawString(Text[0], payfn1, paybr, 10, 6);
                }
                else
                {
                    gr.DrawString(Text[0], payfn3, paybr, 10, 14);
                }
            }
            if (Text[1].Length > 0)
            {
                gr.DrawString(Text[1], payfn2, paybr, 10, 34);
            }

            TVout.TvDrawImage(bmp, 0, 0, m_width , 60);
        }
    }
}
