﻿using System;
using System.Threading;
using System.Text;

namespace ACLAS
{
    public static class TELCAT_IOCTL  
    {
        public static readonly uint READ_DATA = Device.CTL_CODE(FILE_DEVICE.FILE_STREAM,0x100, FILE_METHOD.BUFFERED, FILE_ACCESS.ANY);
        public static readonly uint GET_RCV_PACKET_LEN = Device.CTL_CODE(FILE_DEVICE.FILE_STREAM,0x101, FILE_METHOD.BUFFERED, FILE_ACCESS.ANY); 
    }

    public class TelCatEventArgs : EventArgs
    {
        public byte line;
        public string PhoneNumber;
    }

    public delegate void TelCatCallBack(TelCatEventArgs args);

    public class TelCatClass
    {
        private Device telcat = null;
        private byte[] m_buf;
        private Thread m_thd = null;
        private bool m_run = true;

        public event EventHandler OnRingDetect = null;
        public event EventHandler OnCallIn = null;

        public TelCatClass()
        {
            telcat = new Device("CAT1:", WINAPI.GENERIC_WRITE | WINAPI.GENERIC_READ);
        }

        public bool open()
        {
            bool rc;
            byte [] buf = new byte[4];
            int rsize = 0;

            rc = telcat.Open();

            if (rc)
            {
                rc = telcat.IOControl((int)TELCAT_IOCTL.GET_RCV_PACKET_LEN, null, 0, buf, 4, ref rsize);
                if (rc)
                {
                    m_buf = new byte[CovTool.toInt(buf,0)];

                    m_thd = new Thread(mainproc);
                    m_thd.Start(); 
                }
            }
            return rc;
        }

        public void Close()
        {
            m_run = false;
            telcat.Close();
        }

        private String toString(Byte[] buf, int index, int len)
        {
            StringBuilder s = new StringBuilder();

            for (int i = 0; i < len; i++)
            {
                Byte b = buf[index + i];
                if (b==0x0d)
                    break;
                s.Append((Char)b);
            }
            return s.ToString();
        }

        private void mainproc()
        {
            bool rc;
            int rsize=0;

            while (true)
            {
                rc = telcat.IOControl((int)TELCAT_IOCTL.READ_DATA, null, 0, m_buf, m_buf.Length, ref rsize);

                if (!m_run) break;

                if (rc)
                {
                    TelCatEventArgs args = new TelCatEventArgs();

                    args.line  = (byte)(m_buf[3] - '0');

                    if (rsize == 5)
                    {
                        if (m_buf[1] == 0xa1 && m_buf[4] == 'R') // ring
                        {
                            if (OnRingDetect != null)
                            {
                                OnRingDetect.Invoke(this, args);
                            }
                        }
                    }
                    else
                    {
                        if (m_buf[1] == 0xa0)
                        {
                            args.PhoneNumber = toString(m_buf, 4, 20);
                            if (OnCallIn != null)
                            {
                                OnCallIn.Invoke(this, args);   
                            }
                        }
                    }
                }
            }
        }


    }
}
