﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;
using System.Threading;

namespace ACLAS
{
    public class iButtonClass
    {
        // POLYNOMIAL = X8 + X5 + X4 + 1
        private Byte[] crctbl = new Byte[256];
        private Device iButton = null;
        private Thread thd = null;
        private Byte[] buf = new Byte[9];
        public  Boolean Connect = false;

        public event EventHandler OniButtonRead = null;
        public event EventHandler OniButtonDisconnect = null;

        private ManualResetEvent Enable = new ManualResetEvent(false);

        public UInt16[] SoundOK = new UInt16[]
        {
              1 , 3400, 100 , 40 
        };

        public UInt16[] SoundReady = new UInt16[]
        {
               2 , 1600, 60 , 40 ,
               1 , 2400, 200 , 40 
        };

        public iButtonClass()
        {
            // 打開驅動裝置類別
            iButton = new Device("HDQ1:", WINAPI.GENERIC_WRITE | WINAPI.GENERIC_READ);
            int acc;
            int crc;

            for (int i = 0; i < 256; i++)
            {
                acc = i;
                crc = 0;

                for (int j = 0; j < 8; j++)
                {
                    if (((acc ^ crc) & 0x01) == 0x01)
                    {
                        crc = ((crc ^ 0x18) >> 1) | 0x80;
                    }
                    else
                        crc = crc >> 1;

                    acc = acc >> 1;
                }

                crctbl[i] = (Byte)crc;
            }
        }

        public Boolean Open()
        {
            Boolean rc;

            rc = iButton.Open();

            if (rc)
            {
                thd = new Thread(iButtonProc);
                thd.Start();
            }
            return rc;
        }

        public void Close()
        {
            if (thd != null)
            {
                thd.Abort();
                Thread.Sleep(200);
            }
            iButton.Close();
        }

        public void Start()
        {
            Enable.Set();
        }

        public void Stop()
        {
            Enable.Reset();
        }

        private Boolean ReadSerialNo()
        {
            int rsize;
            Byte data;

            buf[0] = 0x33;
            for (int i = 0; i < 5; i++)
            {
                rsize = iButton.Read(buf, 9);
                if (rsize == 9)
                    break;
            }

            data = 0xff;

            for (int i = 7; i > 1; i--)
            {
                data &= buf[i];
            }

            return (data != 0xff);
        }

        public String SerialNumber
        {
            get
            {
                Int64 val = 0;

                for (int i = 7; i > 1; i--)
                {
                    val = val * 256 + buf[i];
                }
                return val.ToString("D15");
            }
        }

        public Byte FamilyCode
        {
            get
            {
                return buf[1];
            }
        }

        private void iButtonProc()
        {
            Boolean rc;

            while (true)
            {
                Enable.WaitOne();

                rc = ReadSerialNo();

                if (Connect)
                {
                    if (!rc)
                    {
                        Connect = false;
                        Buzzer.SoundOut(SoundReady);
                        if (OniButtonDisconnect != null)
                        {
                            OniButtonDisconnect.Invoke(this, null);
                        }
                    }
                }
                else
                {
                    if (rc)
                    {
                        Connect = true;
                        Buzzer.SoundOut(SoundOK);
                        if (OniButtonRead != null)
                        {
                            OniButtonRead.Invoke(this, null);
                        }
                    }
                }
                Thread.Sleep(50);
            }
        }
    }
}
