﻿using System;
using System.Drawing;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Globalization;
using System.Threading;

namespace PowerOff
{
    public partial class Form1 : Form
    {
        private Device PwrDrawer = new Device("PKD1:", WINAPI.GENERIC_WRITE | WINAPI.GENERIC_READ);   // 打開驅動裝置類別

        [DllImport("coredll.dll")]
        public static extern bool KernelIoControl(
            UInt32 dwIoControlCode,
            IntPtr lpInBuf,
            UInt32 nInBufSize,
            IntPtr lpOutBuf,
            UInt32 nOutBufSize,
            IntPtr lpBytesReturned);

        const UInt32 knIOCTRL_REBOOT = 0x0101003C;

        CultureInfo ci = CultureInfo.CurrentCulture;

        string Charging = "電池充電中";
        string Supplying = "電池供電中";
        string Full = "電池已充飽";

        private string btInfo, btExit;

        public Form1()
        {
            InitializeComponent();
        }

        private void WindowLoction()
        {
            Location = new Point((Screen.PrimaryScreen.WorkingArea.Width - Width) / 2, (Screen.PrimaryScreen.WorkingArea.Height - Height) / 2);
        }

        private void Form1_Load(object sender, EventArgs e)
        {
          
            PwrDrawer.Open();  
                       
            switch (ci.LCID)
            {
                case 0x0804:
                    button1.Text = "关机";
                    btReboot.Text = "重启";
                    btCancel.Text  = "取消";
                    btBatteryInfo.Text = "电池信息";
                    btExit = "回到主菜单";
                    lbVolt.Text = "当前电压";
                    lbTemp.Text = "当前温度";
                    lbDevice.Text = "型号";
                    lbState.Text = "电量百分比";
                    lbDVolt.Text = "充饱电压";
                    lbDCap.Text = "充饱电量";
                    lbCap.Text = "当前电量";
                    lbMaxCap.Text = "最大电量";
                    lbAvgCurrent.Text = "平均电流";
                    lbCurrent.Text = "当前电流";
                    Charging = "电池充电中";
                    Supplying = "电池供电中";
                    Full = "电池已充饱";
                    break;
                case 0x0404:
                    btExit = "關閉電池訊息";
                    break;
                default:
                    button1.Text = "Power OFF";
                    btReboot.Text = "Reboot";
                    btCancel.Text = "Cancel";
                    btBatteryInfo.Text = "Battery Info";
                    btExit = "Close Info";
                    lbVolt.Text = "Voltage";
                    lbTemp.Text = "Tempareture";
                    lbDevice.Text = "Device Name";
                    lbState.Text = "Percentage";
                    lbDVolt.Text = "Design Voltage";
                    lbDCap.Text = "Design Capacity";
                    lbCap.Text = "Current Capacity";
                    lbMaxCap.Text = "Full Capacity";
                    lbAvgCurrent.Text = "Average Current";
                    lbCurrent.Text = "Current";
                    Charging = "Battery Charging";
                    Supplying = "Battery Supplying";
                    Full = "Battery Full";
                    break;
            }

            btInfo = btBatteryInfo.Text;

            panelInfo.Visible = false;

            if (PwrDrawer.GetBatteryInfo())
            {
                Size = new Size(304, Size.Height);

                tbDevice.Text = PwrDrawer.BatteryInfo.DeviceName;
                tbMaxCap.Text = PwrDrawer.BatteryInfo.FullChargeCapacity + " mAh";
                tbDCap.Text = PwrDrawer.BatteryInfo.DesignCapacity + " mAh";
                tbDVolt.Text = (PwrDrawer.BatteryInfo.DesignVoltage * 2) + " mV";
            }
            else
            {
                btBatteryInfo.Visible = false;
                btCancel.Location = new Point(btCancel.Location.X, btBatteryInfo.Location.Y);
                Size = new Size(304,304);  
            }

            WindowLoction();
        }

        string ChargeStatus()
        {
            int current = PwrDrawer.BatteryInfo.Current;
            return (current == 0) ? Full : (current > 0) ? Charging : Supplying;
        }

        void UpdateInfo()
        {
            PwrDrawer.GetBatteryInfo();
            int tmp = PwrDrawer.BatteryInfo.Tempareture;
            tbVolt.Text = PwrDrawer.BatteryInfo.Voltage + " mV";
            tbCap.Text = PwrDrawer.BatteryInfo.RemainningCapacity + " mAh";
            tbTemp.Text = tmp / 10 + "." + tmp % 10 + " C";
            tbState.Text = PwrDrawer.BatteryInfo.AbsoluteStateOfChage + " %";
            lbCurMode.Text = ChargeStatus();
            tbCurrent.Text = Math.Abs(PwrDrawer.BatteryInfo.Current) + " mA";
            tbAvgCurrent.Text = Math.Abs(PwrDrawer.BatteryInfo.AvgCurrent) + " mA";
        }

        private void reboot()
        {
            PwrDrawer.ReBoot();  
        }

        private void button2_Click(object sender, EventArgs e)
        {
            PwrDrawer.Close();  
            Close();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            PwrDrawer.ShutdownPower(); 
        }

        private void btReboot_Click(object sender, EventArgs e)
        {
            reboot();
        }

        private void btBatteryInfo_Click(object sender, EventArgs e)
        {
            if (panelInfo.Visible)
            {
                panelInfo.Visible = false;
                btBatteryInfo.Text = btInfo;
                Size = new Size(304, Size.Height);
                timer1.Enabled = false; 
            }
            else
            {
                UpdateInfo();
                panelInfo.Visible = true;
                btBatteryInfo.Text = btExit;
                Size = new Size(608, Size.Height);
                timer1.Enabled = true;
            }
            WindowLoction(); 
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            UpdateInfo();
        }
    }

    // 裝置類別 by Jacky
    public class Device
    {
        [StructLayout(LayoutKind.Sequential)]
        public struct HY4231_Info
        {
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 10)]
            public String DeviceName;
            public UInt16 Tempareture;
            public UInt16 Voltage;
            public UInt16 RemainningCapacity;
            public UInt16 FullChargeCapacity;
            public UInt16 AbsoluteStateOfChage;
            public UInt16 DesignCapacity;
            public UInt16 DesignVoltage;
            public Int16  Current;
            public Int16  AvgCurrent;
        }

        public HY4231_Info BatteryInfo = new HY4231_Info(); 
        
        [DllImport("coredll.dll")]
        private static extern uint SetDevicePower(
        String Device,
        uint flag,
        uint mode
        );


        [DllImport("coredll.dll")]
        public static extern bool WriteFile(
        System.IntPtr h_comm,
        Byte[] text,
        int length,
        ref   int length2,
        int overlap
        );

        [DllImport("coredll.dll")]
        public static extern bool ReadFile(
        System.IntPtr h_comm,
        Byte[] text,
        System.UInt32 length,
        ref   int length2,
        int overlap
        );

        [DllImport("coredll.dll")]
        public static extern bool DeviceIoControl(
        System.IntPtr h_comm,
        int code,
        Byte[] inBuf,
        int InBufSize,
        Byte[] outBuf,
        int outBufSize,
        ref int BytesReturned,
        int overlap
        );

        [DllImport("coredll.dll")]
        public static extern bool DeviceIoControl(
        System.IntPtr h_comm,
        int code,
        Byte[] inBuf,
        int InBufSize,
        ref  HY4231_Info outBuf,
        int outBufSize,
        ref int BytesReturned,
        int overlap
        );



        [DllImport("coredll.dll")]
        public static extern System.IntPtr CreateFile(
        string FileName,                                         //   file   name 
        uint DesiredAccess,                                   //   access   mode 
        uint ShareMode,                                           //   share   mode 
        uint SecurityAttributes,                         //   Security   Attributes 
        uint CreationDisposition,                       //   how   to   create 
        uint FlagsAndAttributes,                         //   file   attributes 
        int hTemplateFile                                       //   handle   to   template   file 
        );

        [DllImport("coredll.dll")]
        public static extern bool CloseHandle(System.IntPtr h_comm);

        private String DeviceName;
        private System.IntPtr Handle;
        private uint DeviceMode;
        private bool isOpen = false;

        public Device(String name, uint mode)
        {
            DeviceName = name;
            DeviceMode = (uint)mode;
        }

        public void ShutdownPower()
        {
            uint rc = SetDevicePower(DeviceName, WINAPI.POWER_NAME, (uint)WINAPI.POWER_STATE.D4);
        }

        public void ReBoot()
        {
            int rsize = 0;
            DeviceIoControl(Handle, 6 , null , 0 , null , 0 , ref rsize, 0);
        }

        public Boolean IOControl(int code, Byte[] inbuf, int insize, Byte[] obuf, int osize, ref int rsize)
        {
            rsize = 0;
            return DeviceIoControl(Handle, code, inbuf, insize, obuf, osize, ref rsize, 0);
        }

        public Boolean Open()
        {
            if (isOpen)
            {
                Close();
            }
            Handle = CreateFile(DeviceName, DeviceMode, 0, 0, WINAPI.OPEN_EXISTING, 0, 0);
            isOpen = ((int)Handle != WINAPI.INVALID_HANDLE_VALUE);
            return isOpen;
        }

        public Boolean Close()
        {
            isOpen = false;
            return CloseHandle(Handle);
        }

        public bool Write(Byte[] buf, int size)
        {
            int rsize = 0;
            return WriteFile(Handle, buf, size, ref rsize, 0);

        }

        public bool GetBatteryInfo()
        {
            int rsize = 0;
            bool rc = DeviceIoControl(Handle,81, null, 0, ref BatteryInfo, Marshal.SizeOf(BatteryInfo), ref rsize, 0);
            return rc;
        }
    }

    public static class WINAPI
    {
        public const UInt32 OPEN_EXISTING = 3;
        public const UInt32 GENERIC_READ = 0x80000000;
        public const UInt32 GENERIC_WRITE = 0x40000000;
        public const Int32 INVALID_HANDLE_VALUE = -1;
        public const uint POWER_NAME = 1;
        public enum POWER_STATE : uint
        {
            D0 = 0,
            D1,
            D2,
            D3,
            D4
        }
    }
}