﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Threading;

namespace Demo
{
    public class RFIDDriver
    {
        public enum WorkMode
        {
            End,
            WaitCard,
            DetectCard,
            Idle
        }

        public enum WorkType
        {
            GetCardNo   // 讀取卡號
        }

        private Form1 pForm;
        private const int DataLen = 26;
        private Byte[] inBuf = new Byte[DataLen];
        private Byte[] OutBuf = new Byte[DataLen];
        private int rsize = 0;
        private Device RfidCard = new Device("RID1:", 0);   // 打開驅動裝置類別
      
        Thread  thdRFID = null;
        private WorkMode WorkStatus = WorkMode.Idle;
        private WorkType DoStatus = WorkType.GetCardNo;  
        private Byte[] TagType = null;
        private String CardNo; // 卡號
        private String CardType;
        private Byte[] SNR = null;
        private Byte CardSize;

        private UInt16[] RFIDSound = new UInt16[] {
            1 , 2600, 100 , 40 ,
        };

        private UInt16[] RFIDReady = new UInt16[] {
            2 , 2000, 60 , 40 ,
            1 , 3000, 200 , 40 
        };

        public RFIDDriver(Form1 pf)
        {
            pForm = pf;
        }

        public Boolean Open()
        {
            return RfidCard.Open();
        }

        public void Start(WorkMode mode,WorkType type)
        {
            if (WorkStatus != WorkMode.Idle)
                Abort(); 
            if (thdRFID==null) 
               thdRFID = new Thread(RFIDMain);
            WorkStatus = mode;

            type = WorkType.GetCardNo;

            thdRFID.Start();
        }

        private void Abort()
        {
            if (thdRFID != null)
            {
                thdRFID.Abort();
                Thread.Sleep(100);
                WorkStatus = WorkMode.Idle;
                thdRFID = null;
                TurnOff();   // Turn off RDIF 
                pForm.ShowLog("RFID.Abort()\r\n"); 
            }
        }

        public void Close()
        {
            Abort();
            RfidCard.Close();
        }

        private void RFIDMain()
        {
            Boolean Abort = false;

            while (!Abort)
            {
                switch (WorkStatus)
                {
                    case WorkMode.End:
                        Abort = true;
                        break;
                    case WorkMode.WaitCard:

                        Buzzer.SoundOut(RFIDReady);   

                        while (ReadCardNoA() != RetCode.MI_OK);

                        printf(CardType + " Card , Card No :" + CardNo+" Size : 0x"+CardSize.ToString("X2") + "\r\n");

                        Buzzer.SoundOut(RFIDSound);    

                        // 連續 5 次都找不到卡片 , 以確定卡片已經移出感應區 , 以防止連續操作卡片. 例如重覆扣卡
                        for (int i = 0; i < 5; i++)
                        {
                            if (ReadCardNoA() == RetCode.MI_OK)
                                i = 0;
                        }

                        printf("RFID Card Leave!\r\n");

                        break;
                    default:
                        printf("Unknow Work Status : " + ((int)WorkStatus) + "\r\n");
                        break;
                }
            }

            WorkStatus = WorkMode.Idle;
            thdRFID = null;
        }

        // inBuf and OutBuf Struct Toatl 26 Bytes
        // 0 BYTE pkg_no;
        // 1 BYTE cmd;          -> Command
        // 2 BYTE len;          -> Length  
        // 3 BYTE dat[22];      -> Data
        // BYTE bcc;

        public RetCode Command(CommandCode code, Byte len, Byte[] buf)
        {
            RetCode rc;
            inBuf[0] = 0;  // reserve 
            inBuf[1] = (Byte)code;
            inBuf[2] = len;
            if (len > 22) return RetCode.MI_BYTECOUNTERR;  // input data length too big
            for (int i = 0; i < len; i++)
                inBuf[i + 3] = buf[i];
            rsize = 0;
            if (RfidCard.IOControl((int) IOCtrl.ReadCard, inBuf, DataLen, OutBuf, DataLen, ref rsize))
                rc = (RetCode)OutBuf[1];
            else
                rc = RetCode.MI_UNKNOWN_COMMAND;  // Command Error
            return rc;
        }

        public void  TurnOff()
        {
            rsize = 0;
            RfidCard.IOControl((int)IOCtrl.Turn_OFF , null ,  0 , null , 0, ref rsize);
        }

        public Boolean  GetInfo(ref int SerialNo,ref String ProductClassCode,ref String DriverVersion)
        {
            Boolean rc;

            rc = (mifs_set_config() == RetCode.MI_OK);

            if (rc)
            {
                rc = (Command(CommandCode.GetInfo, 0, null) == RetCode.MI_OK);
                if (rc)
                {
                    ProductClassCode = CovTool.HexString(OutBuf, 3, 5);
                    SerialNo = CovTool.toInt(OutBuf, 8);
                    DriverVersion = "Ver:" + (OutBuf[12] >> 4) + "." + (OutBuf[12] & 0xf);
                }
            }
            return rc;
        }

        private void printf(String msg)
        {
            pForm.ShowLog(msg);  
        }

        private RetCode mifs_set_config()
        {
            return Command(CommandCode.Config,0,null);
        }

        private RetCode mifs_Anticoll2(Byte Bcnt)
        {
            RetCode rc;
            Byte[] buf = new Byte[1];
            buf[0] = Bcnt;
            rc = Command(CommandCode.Anticoll2, 1, buf);
            if (rc == RetCode.MI_OK)
            {
                int len = OutBuf[2];
                if (len < 4) len = 4;
                SNR = new Byte[len]; 
                for (int i=0;i<len;i++)
                    SNR[i] = OutBuf[3+i];
            }
            return rc;
        }

        private RetCode mifs_request(Byte mode)
        {
            RetCode rc;
            Byte[] buf = new Byte[1];

            buf[0] = mode;
            rc = Command(CommandCode.Request, 1, buf);

            if (rc != RetCode.MI_OK)
            {
                //printf("read card error code : " + rc.ToString("D3") + "\r\n");
            }
            else
            {
                int len = OutBuf[2];
                TagType = new Byte[len];
                for (int i = 0; i < len; i++)
                    TagType[i] = OutBuf[3 + i];
            }
            return rc;
        }

        private RetCode ReadCardNoA()
        {
            RetCode rc;

            rc = mifs_set_config();

            //   printf("rc = "+ +"\r\n");//test las

            if (rc != RetCode.MI_OK)
            {
                // printf("Error! RFID config Error Code : " + rc.ToString("X2") + "\r\n");
                return rc;
            }

            rc = mifs_request(1);    // 41 laslas

            if (rc != RetCode.MI_OK)
            {
                return rc;
            }

            rc = mifs_Anticoll2(0);   // 防止碰撞

            if (rc != RetCode.MI_OK)
            {
                return rc;
            }

            // 記錄下卡號
            CardNo = "";
            for (int i = 0; i < 4; i++)
                CardNo += SNR[i].ToString("X2");

            CardType = "unknow";
            if (TagType[1] == 0)
            {
                if (TagType[0] == 4)
                    CardType = "S50";
                if (TagType[0] == 2)
                    CardType = "S70";
            }
     
            rc = mifs_select(); // 選擇卡片

            return rc;
        }

        private RetCode mifs_select()
        {
            RetCode rc = Command(CommandCode.Select,4,SNR);
            if (rc == RetCode.MI_OK)
            {
                int len = OutBuf[2];
                CardSize = OutBuf[3];
            }
            return rc;
        }

        public enum IOCtrl
        {
             ReadCard = 1 ,
             Turn_OFF = 2
        }

        public enum RetCode  // Return Code
        {
            MI_OK = 0,
            MI_CHK_OK = 0,
            MI_CRC_ZERO = 0,
            MI_CRC_NOTZERO = 1,
            MI_NOTAGERR = 1,
            MI_CHK_FAILED = 1,
            MI_CRCERR = 2,
            MI_CHK_COMPERR = 2,
            MI_EMPTY = 3,
            MI_AUTHERR = 4,
            MI_PARITYERR = 5,
            MI_CODEERR = 6,

            MI_SERNRERR = 8,
            MI_KEYERR = 9,
            MI_NOTAUTHERR = 10,
            MI_BITCOUNTERR = 11,
            MI_BYTECOUNTERR = 12,
            MI_IDLE = 13,
            MI_TRANSERR = 14,
            MI_WRITEERR = 15,
            MI_INCRERR = 16,
            MI_DECRERR = 17,
            MI_READERR = 18,
            MI_OVFLERR = 19,
            MI_POLLING = 20,
            MI_FRAMINGERR = 21,
            MI_ACCESSERR = 22,
            MI_UNKNOWN_COMMAND = 23,
            MI_COLLERR = 24,
            MI_RESETERR = 25,
            MI_INITERR = 25,
            MI_INTERFACEERR = 26,
            MI_ACCESSTIMEOUT = 27,
            MI_NOBITWISEANTICOLL = 28,
            MI_QUIT = 30,
            MI_RECBUF_OVERFLOW = 50,
            MI_SENDBYTENR = 51,

            MI_SENDBUF_OVERFLOW = 53,
            MI_BAUDRATE_NOT_SUPPORTED = 54,
            MI_SAME_BAUDRATE_REQUIRED = 55,

            MI_WRONG_PARAMETER_VALUE = 60,

            MI_BREAK = 99,
            MI_NY_IMPLEMENTED = 100,
            MI_NO_MFRC = 101,
            MI_MFRC_NOTAUTH = 102,
            MI_WRONG_DES_MODE = 103,
            MI_HOST_AUTH_FAILED = 104,

            MI_WRONG_LOAD_MODE = 106,
            MI_WRONG_DESKEY = 107,
            MI_MKLOAD_FAILED = 108,
            MI_FIFOERR = 109,
            MI_WRONG_ADDR = 110,
            MI_DESKEYLOAD_FAILED = 111,

            MI_WRONG_SEL_CNT = 114,

            MI_WRONG_TEST_MODE = 117,
            MI_TEST_FAILED = 118,
            MI_TOC_ERROR = 119,
            MI_COMM_ABORT = 120,
            MI_INVALID_BASE = 121,
            MI_MFRC_RESET = 122,
            MI_WRONG_VALUE = 123,
            MI_VALERR = 124,
            MI_SAKERR = 0x46	//added by robbie
        }

        public enum CommandCode
        {
            Request = 0x41,
            Anticoll = 0x42,
            Anticoll2 = 0x71,
            CascAnticoll = 0x74,
            Select = 0x43,
            CascSelect = 0x75,
            Authentication = 0x44,
            Authentication2 = 0x72,
            AuthKey = 0x73,
            Halt = 0x45,
            Read = 0x46,
            Write = 0x47,
            ULWrite = 0x76,
            Increment = 0x48,
            Decrement = 0x49,
            Resore = 0x4A,
            Transfer = 0x4B,
            Value = 0x70,
            ValueDebit = 0x77,
            LoadKey = 0x4C,
            Reset = 0x4E,
            GetInfo = 0x4F,
            SetControlBit = 0x50,
            ClrControlBit = 0x51,
            Config = 0x52,
            WriteReg1 = 0x3D,
            ReadReg1 = 0x3E,
            Close = 0x3F,
            CheckWrite = 0x53,
            Buzzer = 0x60,
            ReadE2 = 0x61,
            WriteE2 = 0x62
        }
    }
}
