﻿using System;
using System.Drawing;
using System.Windows.Forms;
using System.Threading;
using System.Runtime.InteropServices;
using ACLAS;

// This Demo is for test IMX6Q cortex-A9 multi-core System 

namespace SMPDemo
{
    public partial class Form1 : Form
    {
        // 打開驅動裝置類別
        private PowerClass PwrDrawer = new PowerClass();
        private System.Windows.Forms.Timer tmr = new  System.Windows.Forms.Timer();

        private Thread thd1 = null;
        private Thread thd2 = null;
        private Thread thd3 = null;
        private Thread thd4 = null;

        private ManualResetEvent cpu1Event = new ManualResetEvent(false);
        private ManualResetEvent cpu2Event = new ManualResetEvent(false);
        private ManualResetEvent cpu3Event = new ManualResetEvent(false);
        private ManualResetEvent cpu4Event = new ManualResetEvent(false); 

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            if (!PwrDrawer.Open())
            {
                MessageBox.Show("fail to open Power Driver!");
                Close();
                return;
            }

            lbCpus.Text = WINAPI.GetCPUs().ToString();

            tmr.Tick += new EventHandler(tmr_Tick);
            tmr.Interval = 1000;
            tmr.Enabled = true;
            
            lbCPUSpeed.Text = PwrDrawer.GetCpuSpeed() + " MHz"; 

            thd1 = new Thread(CPU1Thread);
            thd1.Start();

            thd2 = new Thread(CPU2Thread);
            thd2.Start();

            thd3 = new Thread(CPU3Thread);
            thd3.Start();

            thd4 = new Thread(CPU4Thread);
            thd4.Start();

            Location = new Point((Screen.PrimaryScreen.WorkingArea.Width - Width) / 2, (Screen.PrimaryScreen.WorkingArea.Height - Height) / 2); 
        }

        void tmr_Tick(object sender, EventArgs e)
        {
           byte [] idle = new byte[4];

           byte temp = PwrDrawer.GetCpuTemp();
           if (temp != 0)
           {
               lbTemp.Text = temp.ToString();
           }

           int cnt = PwrDrawer.GetCpuIdle(idle);

           for (int i = 0; i < cnt; i++)
           {
               switch (i)
               {
                   case 0:
                       lbUse1.Text = idle[0] + "%";
                       break;
                   case 1:
                       lbUse2.Text = idle[1] + "%";
                       break;
                   case 2:
                       lbUse3.Text = idle[2] + "%";
                       break;
                   case 3:
                       lbUse4.Text = idle[3] + "%";
                       break;
               }
           }
        }

        private void Form1_Closed(object sender, EventArgs e)
        {
            tmr.Enabled = false;

            if (thd1 != null)
            {
                cpu1Event.Reset();  
                thd1.Abort();
            }
            if (thd2 != null)
            {
                cpu2Event.Reset();  
                thd2.Abort();
            }
            if (thd3 != null)
            {
                cpu3Event.Reset();  
                thd3.Abort();
            }
            if (thd4 != null)
            {
                cpu4Event.Reset();  
                thd4.Abort();
            }
            Thread.Sleep(100);

            PwrDrawer.Close(); 
        }

        void CPU1Thread()
        {
            int k=0;

            // 此線程在 CPU 1 上跑
            // This thread is use CPU1
            WINAPI.SetThreadCPU(1);

            int cpu = WINAPI.GetThreadCPU();

            if (cpu != 1)
            {
                return;
            }
            
            while (true)
            {
                for (int i = 0; i < 1000; i++)
                {
                    cpu1Event.WaitOne();  
                    for (int j = 0; j < 100; j++)
                    {
                        k++;
                    }
                }
                Thread.Sleep(10);
                if (k == 0)
                {
                }
            }
        }

        void CPU2Thread()
        {
            int k = 0;

            // 此線程在 CPU 2 上跑
            // This thread is use CPU2
            WINAPI.SetThreadCPU(2);

            int cpu = WINAPI.GetThreadCPU();

            if (cpu != 2)
            {
                return;
            }

            while (true)
            {
                for (int i = 0; i < 1000; i++)
                {
                    cpu2Event.WaitOne();
                    for (int j = 0; j < 100; j++)
                    {
                        k++;
                    }
                }
                Thread.Sleep(10);
                if (k == 0)
                {
                }
            }
        }

        void CPU3Thread()
        {
            int k = 0;

            // 此線程在 CPU 3 上跑
            // This thread is use CPU3
            WINAPI.SetThreadCPU(3);

            int cpu = WINAPI.GetThreadCPU();

            if (cpu != 3)
            {
                return;
            }

            while (true)
            {
                for (int i = 0; i < 1000; i++)
                {
                    cpu3Event.WaitOne();
                    for (int j = 0; j < 100; j++)
                    {
                        k++;
                    }
                }
                Thread.Sleep(10);
                if (k == 0)
                {
                }
            }
        }

        void CPU4Thread()
        {
            int k = 0;

            // 此線程在 CPU 4 上跑
            // This thread is use CPU4
            WINAPI.SetThreadCPU(4);

            int cpu = WINAPI.GetThreadCPU();

            if (cpu != 4)
            {
                return;
            }

            while (true)
            {
                for (int i = 0; i < 1000; i++)
                {
                    cpu4Event.WaitOne();
                    for (int j = 0; j < 100; j++)
                    {
                        k++;
                    }
                }
                Thread.Sleep(10);
                if (k == 0)
                {
                }
            }
        }

        private void btExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void btCpuOn1_Click(object sender, EventArgs e)
        {
            cpu1Event.Set(); 
        }

        private void btCpuOff1_Click(object sender, EventArgs e)
        {
            cpu1Event.Reset(); 
        }

        private void btCpuOn2_Click(object sender, EventArgs e)
        {
            cpu2Event.Set(); 
        }

        private void btCpuOff2_Click(object sender, EventArgs e)
        {
            cpu2Event.Reset(); 
        }

        private void btCpuOn3_Click(object sender, EventArgs e)
        {
            cpu3Event.Set(); 
        }

        private void btCpuOff3_Click(object sender, EventArgs e)
        {
            cpu3Event.Reset(); 
        }

        private void btCpuOn4_Click(object sender, EventArgs e)
        {
            cpu4Event.Set(); 
        }

        private void btCpuOff4_Click(object sender, EventArgs e)
        {
            cpu4Event.Reset(); 
        }
    }
}