﻿//#define DebugMode  

using System;
using System.Windows.Forms;
using System.Threading;
using ACLAS.RFID;
using ACLAS;
using ACLAS.Printer;
using System.Drawing;
using System.Globalization;
using test.Properties;
using System.Runtime.InteropServices;

namespace test
{
    public partial class Form1 : Form
    {
        private PowerClass pwr = new PowerClass();
        private CDisplayClass SubLCD = new CDisplayClass();

        private Boolean UseRfid = false;

        private MagClass MagCard = new MagClass();
        private iButtonClass iButton = new iButtonClass();

        private RFIDDriver RFID;
        private SmartCard SCARD_2;

        CultureInfo ci = CultureInfo.CurrentCulture;

        delegate void TextCallBack(String msg);

        int m_Contrast = 3;

        int m_MaxGrad = 8; // Grad Range from 1 to 8

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            SubLCD.Open();
            ShowLog("Open Custom LCD Driver\r\n");
            GetLCDInfo();

            SubLCD.GetLCDContrast(ref m_Contrast);
            SubLCD.GetLCDContrastRange(ref m_MaxGrad);

            AclasPrinter.Open();
            ShowLog("Open Printer Driver\r\n");
            pwr.Open();
            ShowLog("Open Power and Drawer Driver\r\n");

            // 設定當電源狀態改變事件通知
            pwr.OnPwrChange += new EventHandler(pwr_OnPwrChange);

            if (iButton.Open())
            {
                ShowLog("Open iButton Device OK!\r\n");

                iButton.OniButtonDisconnect += new EventHandler(iButton_OniButtonDisconnect);

                iButton.OniButtonRead += new EventHandler(iButton_OniButtonRead);

                iButton.Start();
            }
            else
            {
                ShowLog("!!! Open iButton Device Error !!!\r\n");
            }

            if (MagCard.Open())
            {
                ShowLog("Open Mag Card Device OK!\r\n");

                MagCard.OnMagCardRead += new EventHandler(MagCard_OnMagCardRead);
            }
            else
            {
                ShowLog("Mag Card Device not found!\r\n");
            }

            RFID = new RFIDDriver(tbLog);

            if (RFID.Open())
            {
                ShowLog("Open RFID Card Device OK!\r\n");
                UseRfid = true;
            }
            else
            {
                ShowLog("!!! RFID Card Device not found !!!\r\n");
            }

            SCARD_2 = new SmartCard(2);
            SCARD_2.SetDebugTextBox(tbLog);

            if (SCARD_2.Open())
            {
                ShowLog("Open Smart Card 2 Device OK!\r\n");
            }
            else
            {
                ShowLog("!!! Open Smart Card Device 2 Error !!!\r\n");
            }

            if (Buzzer.Open())
            {
                ShowLog("Open Buzzer Driver OK!\r\n");
            }
            else
            {
                MessageBox.Show("Open Buzzer Device Error!");
                Close();
            }

            String ProductClassCode = "";
            int SerialNo = 0;
            String DriverVersion = "";

            if (UseRfid)
            {
                if (RFID.GetInfo(ref SerialNo, ref ProductClassCode, ref DriverVersion))
                {
                    ShowLog("RFID Reader Product Class Code : 0x" + ProductClassCode + "\r\n");
                    ShowLog("RFID Reader Product Serial Number : " + SerialNo + "\r\n");
                    ShowLog("RFID Reader Driver Version : " + DriverVersion + "\r\n");
                    // 指定讀卡完成事件, 所要呼叫的委派
                    RFID.OnReadDone += new EventHandler(RFID_OnReadDone);
                    RFIDStart();
                }
                else
                {
                    ShowLog("RFID Reader GetInfo() Error!\r\n");
                    UseRfid = false;
                }
            }

            cbPages.Text = "1";

            cbShade.SelectedIndex = 9;

#if DebugMode
            DebugClass.Init(this);  
#endif

        }

        void pwr_OnPwrChange(object sender, EventArgs e)
        {
            switch (pwr.PowerEvent)
            {
                case PowerType.PowerKeyDown:
                    ShowLog("Found Power Button to Turn Off Power\r\n");
                    // 進行關機前所需的工作 , 請注意執行緒安全
                    Thread.Sleep(500);
                    pwr.ShutdownPower();  // 關機
                    break;
                case PowerType.PowerKeyUp:
                    ShowLog("Found Power Button to Turn On Power\r\n");
                    // 剛從睡眠中醒來
                    break;
                case PowerType.PowerLose:
                    // 掉電
                    ShowLog("Power Lose!\r\n");
                    // 進行關機前所需的工作 , 請注意執行緒安全
                    Thread.Sleep(500);
                    pwr.ShutdownPower();  // 關機
                    break;
                case PowerType.PowerRecovery:
                    // 從掉電中回復
                    ShowLog("Power Recovery!\r\n");
                    break;
                case PowerType.PowerNormal:
                    // 正常
                    ShowLog("Power Ready!\r\n");
                    break;
                default:
                    ShowLog("Unkown Power type!\r\n");
                    break;
            }
        }

        private void btBLoff_Click(object sender, EventArgs e)
        {
            SubLCD.BackLightOff();
            ShowLog("Custum LCD Back Light OFF\r\n");
        }

        private void GetLCDInfo()
        {
            int col = 0, row = 0;
            SubLCD.GetLCDInfo(ref col, ref row);
            ShowLog("Custom LCD Size : COL: " + col + "  ROW: " + row + "\r\n");
            if (SubLCD.GraphicsMode)
            {
                ShowLog("This is Graphics Mode Display\r\n");
            }
        }

        private void Form1_Closed(object sender, EventArgs e)
        {
            ShowLog("Close All Driver\r\n===End of Test===\r\n\r\nThank You!");
            AclasPrinter.Shade(0);
            // 關閉所有打開的設備
            SubLCD.Close();
            pwr.Close();
            AclasPrinter.Close();
            iButton.Close();
            MagCard.Close();
            RFID.Close();
            SCARD_2.Close();
            Buzzer.Close();
            Thread.Sleep(1000);
        }

        private void btBLon_Click(object sender, EventArgs e)
        {
            SubLCD.BackLightOn();
            ShowLog("Custom LCD Back Light On\r\n");
        }

        void Str2Byte(string s1, string s2, Byte[] buf, int width)
        {
            int i;
            for (i = 0; i < width * 2; i++) buf[i] = (Byte)' ';
            for (i = 0; i < s1.Length && i < width; i++)
                buf[i] = (byte)s1[i];
            for (i = 0; i < s2.Length && i < width; i++)
                buf[i + width] = (byte)s2[i];
        }

        void Str2Byte(string s1, string s2, string s3, string s4, Byte[] buf, int width)
        {
            int i;
            for (i = 0; i < width * 4; i++) buf[i] = (Byte)' ';
            for (i = 0; i < s1.Length && i < width; i++)
                buf[i] = (byte)s1[i];
            for (i = 0; i < s2.Length && i < width; i++)
                buf[i + width] = (byte)s2[i];
            for (i = 0; i < s3.Length && i < width; i++)
                buf[i + width * 2] = (byte)s3[i];
            for (i = 0; i < s4.Length && i < width; i++)
                buf[i + width * 3] = (byte)s4[i];
        }

        private void btShow1_Click(object sender, EventArgs e)
        {
            ShowLog("Show Demo String 1\r\n");

            if (SubLCD.GraphicsMode)
            {
                SubLCD.Clear();

                switch (SubLCD.Model)
                {
                    case CDisplayModel.COG64x384:

                        SubLCD.gr.DrawRectangle(new Pen(Color.Black, 1), new Rectangle(0, 0, SubLCD.Width - 1, SubLCD.Height - 2));

                        switch (ci.LCID)
                        {
                            case 0x804:
                                SubLCD.gr.DrawString("頂尖科技 64x384 COG Display", new Font("SimSun", 20, FontStyle.Regular), SubLCD.brush, 5 , 18);
                                break;
                            case 0x404:
                                SubLCD.gr.DrawString("頂尖科技 64x384 COG Display", new Font("新細明", 20, FontStyle.Bold), SubLCD.brush, 12, 18);
                                break;
                            default:
                                SubLCD.gr.DrawString("Aclas 64x384 COG Display", new Font("Arail", 20, FontStyle.Regular), SubLCD.brush, 34 , 12);
                                break;
                        }
                        break;


                    case CDisplayModel.LCD32x240:

                        SubLCD.gr.DrawRectangle(new Pen(Color.Black, 1), new Rectangle(0, 0, SubLCD.Width - 1, SubLCD.Height - 2));

                        switch (ci.LCID)
                        {
                            case 0x804:
                                SubLCD.gr.DrawString("頂尖科技 CD5 LCD", new Font("SimSun", 20, FontStyle.Regular), SubLCD.brush, 8, 2);
                                break;
                            case 0x404:
                                SubLCD.gr.DrawString("頂尖科技 CD5 LCD", new Font("新細明", 20, FontStyle.Regular), SubLCD.brush, 8, 2);
                                break;
                            default:
                                SubLCD.gr.DrawString("Aclas CD5 LCD", new Font("Arail", 20, FontStyle.Regular), SubLCD.brush, 30, 0);
                                break;
                        }

                        break;
                    case CDisplayModel.LCD65x264:

                        SubLCD.gr.DrawRectangle(new Pen(Color.Black, 1), new Rectangle(0, 0, SubLCD.Width - 1, SubLCD.Height - 2));

                        switch (ci.LCID)
                        {
                            case 0x804:
                                SubLCD.gr.DrawString("頂尖科技 CD1 LCD", new Font("SimSun", 20, FontStyle.Regular), SubLCD.brush, 18, 2);
                                SubLCD.gr.DrawString("这是一个 LCD 展示程序", new Font("SimSun", 12, FontStyle.Regular), SubLCD.brush, 40, 30);
                                SubLCD.gr.DrawString("ArmPos 客显 LCD 分辨率 65x264", new Font("SimSun", 10, FontStyle.Regular), SubLCD.brush, 30, 47);
                                break;
                            case 0x404:
                                SubLCD.gr.DrawString("頂尖科技 CD1 LCD", new Font("新細明", 20, FontStyle.Regular), SubLCD.brush, 18, 2);
                                SubLCD.gr.DrawString("這是一個 LCD 展示程式", new Font("新細明", 12, FontStyle.Regular), SubLCD.brush, 48, 30);
                                SubLCD.gr.DrawString("ArmPos 客顯 LCD 解析度 65x264", new Font("新細明", 10, FontStyle.Regular), SubLCD.brush, 45, 47);
                                break;
                            default:
                                SubLCD.gr.DrawString("Aclas CD1 LCD", new Font("Arail", 20, FontStyle.Regular), SubLCD.brush, 40, 0);
                                SubLCD.gr.DrawString("This is a LCD test program", new Font("Arail", 12, FontStyle.Regular), SubLCD.brush, 40, 28);
                                SubLCD.gr.DrawString("ArmPos Custom Display LCD 65x264", new Font("Arail", 10, FontStyle.Regular), SubLCD.brush, 30, 45);
                                break;
                        }
                        break;
                    case CDisplayModel.VFD24x128:

                        SubLCD.gr.DrawRectangle(new Pen(Color.Black, 1), new Rectangle(0, 0, SubLCD.Width - 1, SubLCD.Height - 1));

                        switch (ci.LCID)
                        {
                            case 0x804:
                                SubLCD.gr.DrawString("頂尖科技 CD8", new Font("SimSun", 14, FontStyle.Regular), SubLCD.brush, 5, 2);
                                break;
                            case 0x404:
                                SubLCD.gr.DrawString("頂尖科技 CD8", new Font("新細明", 14, FontStyle.Regular), SubLCD.brush, 5, 2);
                                break;
                            default:
                                SubLCD.gr.DrawString("Aclas CD8 VFD", new Font("Arail", 14, FontStyle.Regular), SubLCD.brush, 3, 0);
                                break;
                        }
                        break;
                }
                SubLCD.Show();
            }
            else
            {
                switch (SubLCD.Model)
                {
                    case CDisplayModel.VFD1x9:
                        {
                            String s = "1234567.89";
                            Byte[] buf = new Byte[s.Length];
                            for (int i = 0; i < s.Length; i++)
                                buf[i] = (Byte)s[i];
                            SubLCD.Show(buf, s.Length);
                        }
                        break;
                    case CDisplayModel.LCD32x144:
                        {
                            Byte[] buf = new Byte[36];
                            string s1 = "BankPos WINCE 6.0";
                            string s2 = "Demo By Jacky";
                            Str2Byte(s1, s2, buf, 18);

                            SubLCD.ClrSubLCD();
                            SubLCD.Show(buf, 36);
                        }
                        break;
                    case CDisplayModel.LCD65x132:
                        {
                            Byte[] buf = new Byte[64];
                            string s1 = "WELCOME YOU!";
                            string s2 = "Milk        $100";
                            string s3 = "Sandwich    $20 ";
                            string s4 = "Total:      $120";
                            Str2Byte(s1, s2, s3, s4, buf, 16);
                            SubLCD.ClrSubLCD();
                            SubLCD.Show(buf, 64);
                        }
                        break;
                }
            }
        }

        private void btShow2_Click(object sender, EventArgs e)
        {
            ShowLog("Show Demo String 2\r\n");

            if (SubLCD.Model == CDisplayModel.LCD32x144 || SubLCD.Model == CDisplayModel.LCD65x132)
            {
                SubLCD.SetGraphMode(true);
            }

            if (SubLCD.GraphicsMode)
            {
                SubLCD.Clear();

                switch (SubLCD.Model)
                {
                    case CDisplayModel.LCD32x144:
                        switch (ci.LCID)
                        {
                            case 0x804:  // 簡體中文
                                SubLCD.gr.DrawString("欢迎光临", new Font("SimSun", 20, FontStyle.Regular), SubLCD.brush, 20, 1);
                                break;
                            case 0x404:  // 繁體中文
                                SubLCD.gr.DrawString("歡迎光臨", new Font("新細明", 20, FontStyle.Regular), SubLCD.brush, 20, 1);
                                break;
                            default:
                                SubLCD.gr.DrawString("WELCOME", new Font("Arail", 20, FontStyle.Regular), SubLCD.brush, 13, 0);
                                break;
                        }
                        break;
                    case CDisplayModel.LCD32x240:
                        switch (ci.LCID)
                        {
                            case 0x804:  // 簡體中文
                                SubLCD.gr.DrawString("欢迎光临 谢谢惠顾", new Font("SimSun", 20, FontStyle.Regular), SubLCD.brush, 8, 1);
                                break;
                            case 0x404:  // 繁體中文
                                SubLCD.gr.DrawString("歡迎光臨 謝謝惠顧", new Font("新細明", 20, FontStyle.Regular), SubLCD.brush, 8, 1);
                                break;
                            default:
                                SubLCD.gr.DrawString("WELCOME YOU!", new Font("Arail", 20, FontStyle.Regular), SubLCD.brush, 26, 0);
                                break;
                        }
                        break;

                    case CDisplayModel.COG64x384:
                    case CDisplayModel.LCD65x264:
                        Bitmap bmp = Resources.aclas;
                        SubLCD.gr.DrawImage(bmp, (SubLCD.Width - bmp.Width) / 2, (SubLCD.Height - bmp.Height) / 2);
                        break;

                    case CDisplayModel.VFD24x128:
                        switch (ci.LCID)
                        {
                            case 0x804:  // 簡體中文
                                SubLCD.gr.DrawString("欢迎光临", new Font("SimSun", 18, FontStyle.Regular), SubLCD.brush, 16, 0);
                                break;
                            case 0x404:  // 繁體中文
                                SubLCD.gr.DrawString("歡迎光臨", new Font("新細明", 18, FontStyle.Regular), SubLCD.brush, 16, 0);
                                break;
                            default:
                                SubLCD.gr.DrawString("WELCOME YOU!", new Font("Arail", 12, FontStyle.Regular), SubLCD.brush, 7, 2);
                                break;
                        }
                        break;
                    case CDisplayModel.LCD65x132:

                        SubLCD.gr.DrawRectangle(new Pen(Color.Black, 1), new Rectangle(0, 0, SubLCD.Width - 1, SubLCD.Height - 2));

                        switch (ci.LCID)
                        {
                            case 0x804:
                                SubLCD.gr.DrawString("頂尖科技 CD1", new Font("SimSun", 20, FontStyle.Regular), SubLCD.brush, 10, 2);
                                SubLCD.gr.DrawString("LCD 展示程序", new Font("SimSun", 12, FontStyle.Regular), SubLCD.brush, 20, 30);
                                SubLCD.gr.DrawString("ArmPos 客显 65x132", new Font("SimSun", 10, FontStyle.Regular), SubLCD.brush, 4, 47);
                                break;
                            case 0x404:
                                SubLCD.gr.DrawString("頂尖科技 CD1", new Font("細明體", 14, FontStyle.Regular), SubLCD.brush, 10, 6);
                                SubLCD.gr.DrawString("LCD 展示程式", new Font("新細明", 12, FontStyle.Regular), SubLCD.brush, 20, 30);
                                SubLCD.gr.DrawString("ArmPos 客顯 65x132", new Font("新細明", 10, FontStyle.Regular), SubLCD.brush, 12, 47);
                                break;
                            default:
                                SubLCD.gr.DrawString("Aclas CD1", new Font("Arail", 20, FontStyle.Regular), SubLCD.brush, 7, 0);
                                SubLCD.gr.DrawString("This is a LCD test program", new Font("Arail", 12, FontStyle.Regular), SubLCD.brush, 4, 28);
                                SubLCD.gr.DrawString("ArmPos LCD 65x132", new Font("Arail", 10, FontStyle.Regular), SubLCD.brush, 8, 45);
                                break;
                        }
                        break;
                }
                SubLCD.Show();
            }
            else
            {
                switch (SubLCD.Model)
                {
                    case CDisplayModel.VFD1x9:
                        {
                            String s = "ABCD-EFP,0";
                            Byte[] buf = new Byte[s.Length];
                            for (int i = 0; i < s.Length; i++)
                                buf[i] = (Byte)s[i];
                            SubLCD.Show(buf, s.Length);
                        }
                        break;

                }
            }

            if (SubLCD.Model == CDisplayModel.LCD32x144 || SubLCD.Model == CDisplayModel.LCD65x132)
            {
                SubLCD.SetGraphMode(false);
            }
        }

        private void btContrast2_Click(object sender, EventArgs e)
        {
            switch (SubLCD.Model)
            {
                case CDisplayModel.LCD65x132:
                case CDisplayModel.LCD32x144:
                case CDisplayModel.LCD65x264:
                case CDisplayModel.LCD32x240:
                case CDisplayModel.COG64x384:
                    // Grad Range from 1 to 8
                    if (m_Contrast > 1)
                    {
                        m_Contrast--;
                        ShowLog("Set Contrast Grad: " + m_Contrast + "\r\n");
                        SubLCD.SetLCDContrast(m_Contrast);
                    }
                    break;
                default:
                    ShowLog("VFD Display is not support contrast setup\r\n");
                    break;
            }
        }

        private void btContrast1_Click(object sender, EventArgs e)
        {
            switch (SubLCD.Model)
            {
                case CDisplayModel.LCD65x132:
                case CDisplayModel.LCD32x144:
                case CDisplayModel.LCD65x264:
                case CDisplayModel.LCD32x240:
                case CDisplayModel.COG64x384:
                    if (m_Contrast < m_MaxGrad)
                    {
                        m_Contrast++;
                        ShowLog("Set Contrast Grad: " + m_Contrast + "\r\n");
                        SubLCD.SetLCDContrast(m_Contrast);
                    }
                    break;
                default:
                    ShowLog("VFD Display is not support contrast setup\r\n");
                    break;
            }
        }

        private void btForward_Click(object sender, EventArgs e)
        {
            if (!AclasPrinter.LineFeed(240))
            {
                ShowLog("Printer Line Feed Error!\r\n");
            }
            else
            {
                ShowLog("Line Feed 240 Step...\r\n");
            }
        }

        private void btBackward_Click(object sender, EventArgs e)
        {
            if (!AclasPrinter.BackLineFeed(240))
            {
                ShowLog("Printer Back Line Feed Error!\r\n");
            }
            else
            {
                ShowLog("Back Line Feed 240 Step...\r\n");
            }
        }

        private void SetupShade()
        {
            int shade = cbShade.SelectedIndex - 9;
            AclasPrinter.Shade(shade);
        }

        private void btPrint_Click(object sender, EventArgs e)
        {
            int pages = 1;
            String test;

            ShowLog("Printing......\r\n");

            if (!cbPntContrast.Checked)
            {
                try
                {
                    pages = Convert.ToInt32(cbPages.Text);
                }
                catch
                {
                }

                if (pages == 0 || pages > 200)
                {
                    MessageBox.Show("Pages must form 1 to 200");
                    return;
                }
            }

            SetupShade();

            for (int j = 0; j < pages; j++)
            {
                if (cbPntContrast.Checked)
                {
                    AclasPrinter.PrintString("Page : " + (j + 1).ToString() + "/" + pages + "\r\n");

                    // Printer Shade from -9 (淡) to 15 (濃)
                    for (int i = -9; i < 16; i++)
                    {
                        AclasPrinter.Shade(i);
                        test = "WINCE Printer Shade :" + i.ToString() + "\r\n";
                        AclasPrinter.PrintString(test);
                    }
                }
                else
                {
                    AclasPrinter.PrintString("Page : " + (j + 1).ToString() + "/" + pages + " Shade:" + (cbShade.SelectedIndex - 9).ToString() + "\r\n");

                    if (cbLarge.Checked)
                    {
                        for (int i = 1; i <= 40; i++)
                        {
                            if (AclasPrinter.Width == 576)
                            {
                                test = "This is WINCE TP33 Printer Large DEMO Line:" + i.ToString() + "\r\n";
                            }
                            else
                            {
                                test = "WINCE Print Large DEMO Line:" + i.ToString() + "\r\n";
                            }
                            AclasPrinter.PrintString(test);
                        }
                    }
                    else
                    {
                        for (int i = 1; i <= 5; i++)
                        {
                            if (AclasPrinter.Width == 576)
                            {
                                test = "This is WINCE TP33 Printer Normal DEMO Line :" + i.ToString() + "\r\n";
                            }
                            else
                            {
                                test = "WINCE Printer DEMO Line :" + i.ToString() + "\r\n";
                            }
                            AclasPrinter.PrintString(test);
                        }
                    }
                }

                // 如果帶切刀
                if (AclasPrinter.HaveCutter)
                {
                    // Send Cut Command
                    if (cbHalfCut.Checked)
                    {
                        // Feed 20mm And Half Cut
                        AclasPrinter.HalfCutWithFeed(20);
                    }
                    else
                    {
                        // Feed 20mm And Full Cut
                        AclasPrinter.FullCutWithFeed(20);
                    }
                }
            }

            ShowLog("Waiting for Print!\r\n");

            // Wait until Print OK!
            while (!AclasPrinter.GetPrnStatus())
                Application.DoEvents();

            ShowLog("Print OK!\r\n");

            // 如果不帶切刀
            if (!AclasPrinter.HaveCutter)
            {
                Thread.Sleep(500);
                ShowLog("Line Feed 300 Step...\r\n");
                AclasPrinter.LineFeed(300);
            }
        }

        private void btEnd_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void btDrawer_Click(object sender, EventArgs e)
        {
            // 打開抽屜 1
            pwr.OpenDraw(1);
#if !DebugMode
            ShowLog("Open Drawer 1\r\n");
#endif
        }

        private void btGetID_Click(object sender, EventArgs e)
        {
            string id = "", version = "";

            if (pwr.GetDeviceID(ref version, ref id))
            {
                ShowLog("Hardware Version : v" + version + "\r\n" +
                        "Product ID: [" + id + "]\r\n"
                       );
            }
            else
                ShowLog("Get Product ID Error!\r\n");
        }

        private void btPowerOff_Click(object sender, EventArgs e)
        {
            DialogResult result = MessageBox.Show("Are you sure ?", "WINCE Power OFF", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button1);
            if (result == DialogResult.Yes)
            {
                pwr.ShutdownPower();
            }
        }

        private void btGPRS_Click(object sender, EventArgs e)
        {
            GPRSClass GPRS = new GPRSClass();

            ShowLog("Open GPRS!\r\n");

            if (!GPRS.Connect())
            {
                ShowLog("GPRS Connect Error code:" + GPRS.ErrorCode + "\r\n");
                return;
            }

            ShowLog("GPRS Connect OK!\r\n");

            DialogResult result = MessageBox.Show("Are you sure to disconnect?", "GPRS On Line", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button1);

            if (result == DialogResult.Yes)
            {
                GPRS.Disconnect();
                ShowLog("Close GPRS!\r\n");
            }
        }

        private void btRead_Click(object sender, EventArgs e)
        {
            tbLog.Text = "";
        }

        void iButton_OniButtonRead(object sender, EventArgs e)
        {
            ShowLog("FAMILY CODE:" + iButton.FamilyCode + "\r\n");
            ShowLog("Serial Number:" + iButton.SerialNumber + "\r\n");
        }

        void iButton_OniButtonDisconnect(object sender, EventArgs e)
        {
            ShowLog("iButton disconnect...!\r\n");
        }

        void MagCard_OnMagCardRead(object sender, EventArgs e)
        {
            if (MagCard.Mag1Data != null) ShowLog("Mag Track1:" + MagCard.Mag1Data + "\r\n");
            if (MagCard.Mag2Data != null) ShowLog("Mag Track2:" + MagCard.Mag2Data + "\r\n");
            if (MagCard.Mag3Data != null) ShowLog("Mag Track3:" + MagCard.Mag3Data + "\r\n");
        }

        public void ShowLog(String msg)
        {
            if (tbLog.InvokeRequired)
            {
                TextCallBack tb = new TextCallBack(ShowLog);
                Invoke(tb, new Object[] { msg });
            }
            else
            {
                tbLog.Text += msg;
                tbLog.Select(tbLog.TextLength, 0);
                tbLog.ScrollToCaret();
            }
        }

        private void btExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void RFIDStart()
        {
            RFID.SelectPos(0, 0);
            RFID.ReadMode = RFIDDriver.BlockReadMode.Data;
            RFID.SelectKey(RFIDDriver.KeyType.KeyA, RFID.InitKey);
            RFID.SetLoopMode(true);
            RFID.DoJob(RFIDDriver.JobStatus.Start, RFIDDriver.JobMode.ReadBlock);
            ShowLog("Start RDIF Card to Reading\r\n");
        }

        void RFID_OnReadDone(object sender, EventArgs e)
        {
            ShowLog("Block Data :[" + RFID.GetHexData() + "]\r\n");
        }

        private void btClearLog_Click(object sender, EventArgs e)
        {
            tbLog.Text = "";
        }

        private void cbLarge_CheckStateChanged(object sender, EventArgs e)
        {

        }
    }

#if DebugMode
    public static class DebugClass
    {
        static System.Windows.Forms.Timer tmr;
        static Form1 pForm;
        static Label lbMem;

        public static void Init(Form1 pf)
        {
            pForm = pf;

            lbMem = new Label();
            lbMem.BackColor = Color.Cyan;
            lbMem.Location = new Point(498, 0);
            lbMem.Name = "lbMem";
            lbMem.Size = new Size(100, 20);
            lbMem.Text = "";

            pForm.Controls.Add(lbMem);

            tmr = new System.Windows.Forms.Timer();
            tmr.Tick += new EventHandler(tmr_Tick);
            tmr.Interval = 200;
            tmr.Enabled = true;
        }

        public static void Close()
        {
            tmr.Enabled = false;
        }

        static void tmr_Tick(object sender, EventArgs e)
        {
            lbMem.BringToFront();
            AvailPhys();
        }

        public struct MEMORYSTATUS
        {
            public UInt32 dwLength;
            public UInt32 dwMemoryLoad;
            public UInt32 dwTotalPhys;
            public UInt32 dwAvailPhys;
            public UInt32 dwTotalPageFile;
            public UInt32 dwAvailPageFile;
            public UInt32 dwTotalVirtual;
            public UInt32 dwAvailVirtual;
        }

        [DllImport("CoreDll.dll")]
        public static extern void GlobalMemoryStatus
        (
            ref MEMORYSTATUS lpBuffer
        );

        [DllImport("CoreDll.dll")]
        public static extern int GetSystemMemoryDivision
        (
            ref UInt32 lpdwStorePages,
            ref UInt32 lpdwRamPages,
            ref UInt32 lpdwPageSize
        );

        public static String AvailPhys()
        {
            UInt32 storePages = 0;
            UInt32 ramPages = 0;
            UInt32 pageSize = 0;
            int res = GetSystemMemoryDivision(ref storePages,
                ref ramPages, ref pageSize);

            // Call the native GlobalMemoryStatus method
            // with the defined structure.
            MEMORYSTATUS memStatus = new MEMORYSTATUS();
            GlobalMemoryStatus(ref memStatus);

            String mem = memStatus.dwAvailPhys.ToString();

            lbMem.Text = mem;
            // Show the available memory.
            return mem;
        }
    }
#endif

}